package org.enhydra.demo.wizard.tools;

import java.lang.reflect.Method;
import java.sql.Date;
import java.util.Calendar;
import java.util.HashMap;
import java.util.StringTokenizer;

import org.enhydra.shark.api.RootException;
import org.enhydra.shark.api.client.wfmc.wapi.WMSessionHandle;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.internal.toolagent.AppParameter;
import org.enhydra.shark.api.internal.toolagent.ApplicationBusy;
import org.enhydra.shark.api.internal.toolagent.ApplicationNotDefined;
import org.enhydra.shark.api.internal.toolagent.ApplicationNotStarted;
import org.enhydra.shark.api.internal.toolagent.ToolAgentGeneralException;
import org.enhydra.shark.api.internal.working.CallbackUtilities;
import org.enhydra.shark.toolagent.AbstractToolAgent;
import org.enhydra.shark.xpdl.elements.ExtendedAttribute;
import org.enhydra.shark.xpdl.elements.ExtendedAttributes;

public class StorageTool extends AbstractToolAgent {
	
	/**
	 *  Tool agent for storing data into database. 
	 *  This tool uses default parameters from config file:
	 *  @packageDO - full name og package where is stored DO object
	 *  @table- name of the table
	 *  @checkUnique - check unique indexes for database table
	 *  
	 *  First two parameters will be overwriten with value of extended attribute from xpdl (DataTableClass)
	 *  if they exist. This value is full package name for DO class + DO class name.
	 *  
	 *  This extended attribute must be called "DataTableClass" and its value must be full name of DO Class
	 * 
	 *  @author Svjetlana Milidrag
	 */

	public static final String DATA_TABLE_CLASS_EXT_ATTR_NAME = "DataTableClass";

	private static final String KEY_TABLE = "DatabaseResource.storage.database.table";

	private static final String PACKAGE_KEY = "DatabaseResource.storage.database.packageDO";

	private static final String CHECK_UNIQUE_KEY = "DatabaseResource.storage.database.table.checkunique";

	private static String packageDO = "";

	private static String checkUnique = "";

	private static String table = "";

	private static String fullpackageName = "";

	private Method mth = null;

	private HashMap parameterMap = new HashMap();
	
	/**
	 * Read parameters from config file
	 */

	public void configure(CallbackUtilities c) throws RootException {
		super.configure(c);

		table = c.getProperty(KEY_TABLE);
		packageDO = c.getProperty(PACKAGE_KEY);
		checkUnique = c.getProperty(CHECK_UNIQUE_KEY);
		fullpackageName = packageDO + "." + table + "DO";

	}

	public void invokeApplication(WMSessionHandle _shandle, long _handle,
			WMEntity _appInfo, WMEntity _toolInfo, String _applicationName,
			String _procInstId, String _assId, AppParameter[] _parameters,
			Integer _appMode) throws ApplicationNotStarted,
			ApplicationNotDefined, ApplicationBusy, ToolAgentGeneralException {
		super.invokeApplication(_shandle, _handle, _appInfo, _toolInfo,
				_applicationName, _procInstId, _assId, _parameters, _appMode);

		try {

			status = APP_STATUS_RUNNING;
			
			readParamsFromExtAttributes((String) _parameters[0].the_value);

			store2db();
			
			status = APP_STATUS_FINISHED;

		} catch (Exception e) {
			if (e instanceof ApplicationNotStarted) {
				throw (ApplicationNotStarted) e;
			} else if (e instanceof ApplicationNotDefined) {
				throw (ApplicationNotDefined) e;
			} else if (e instanceof ApplicationBusy) {
				throw (ApplicationBusy) e;
			} else if (e instanceof ToolAgentGeneralException) {
				throw (ToolAgentGeneralException) e;
			} else {
				throw new ToolAgentGeneralException(e);
			}
		}
	}
 /**
  * Function for storing data into database. 
  * 
  * Check for unique key (objectExist(String name) )if in property file parameter which sets @checkUnique has value yes.
  * 
  * All foreign keys should have name with suffix FK in xpdl.  
  * 
  * @throws Exception
  */
	private void store2db() throws Exception {

		String name = fullpackageName;
		String className = "";
		String paramName = "";
		Object paramValue = null;
		String paramMode = "";
		boolean objExist = false;
		Object newDO = null;

		if (checkUnique.equalsIgnoreCase("Yes")) {

			fillTheMap();

			newDO = objectExist(name);

			if (newDO != null)
				objExist = true;
		}

		if (!objExist) {
			Object co = Class.forName(name);
			mth = findMethod(name, "createVirgin");
			newDO = mth.invoke(co, new Object[] {});
		}

		for (int i = 1; i < parameters.length; i++) {

			paramName = parameters[i].the_formal_name;
			paramValue = parameters[i].the_value;
			className = parameters[i].the_class.getName();
			paramMode = parameters[i].the_mode;

			if (paramMode.equalsIgnoreCase("IN") && !paramName.endsWith("FK")
					&& !paramName.equalsIgnoreCase("Unique") && !objExist) {

				if (className.equalsIgnoreCase("java.util.Date")) {
					paramValue = getDate(paramValue);
					className = "java.sql.Date";
				}

				mth = Class.forName(name).getMethod("set" + paramName,
						new Class[] { Class.forName(className) });
				mth.invoke(newDO, new Object[] { (Object) paramValue });

			} else if (paramMode.equalsIgnoreCase("OUT")) {
				mth = findMethod(name, "get_OId");
				parameters[i].the_value = mth.invoke(newDO, new Object[] {})
						.toString();

			}

			if (paramName.endsWith("FK") && !objExist) {
				mth = Class.forName(name).getMethod(
						"oid_set"
								+ paramName
										.substring(0, paramName.length() - 2),
						new Class[] { Class.forName(className) });
				mth.invoke(newDO, new Object[] { (Object) paramValue });

			}

		}

		if (!objExist) {
			mth = findMethod(name, "save");
			mth.invoke(newDO, new Object[] {});
		}


	}

	/**
	 * 
	 * Function that check existing of unique index in database table
	 * 
	 * @param name-DO class full name
	 * @return- DO object if is found or null value
	 * @throws Exception
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 */
	private Object objectExist(String name) throws Exception,
			IllegalAccessException, ClassNotFoundException {

		String uniqueParamValue = "";
		String uniqueName = "";
		String uniqueValue = "";
		String className = name.substring(0, name.length() - 2) + "Query";

		if (parameterMap.containsKey("Unique")) {
			uniqueParamValue = (String) parameterMap.get("Unique");
		}

		StringTokenizer st = new StringTokenizer(uniqueParamValue, ",");
		if (st.countTokens() > 0) {

			while (st.hasMoreTokens()) {

				uniqueName = st.nextToken();

				if (parameterMap.containsKey(uniqueName)) {

					uniqueValue = (String) parameterMap.get(uniqueName);
					Object cl = Class.forName(className).newInstance();

					mth = Class.forName(className).getMethod(
							"setQuery" + uniqueName,
							new Class[] { Class.forName("java.lang.String") });
					mth.invoke(cl, new Object[] { (Object) uniqueValue });
					mth = findMethod(className, "requireUniqueInstance");
					mth.invoke(cl, new Object[] {});
					mth = findMethod(className, "getNextDO");
					Object t = mth.invoke(cl, new Object[] {});
					if (t != null) {
						return t;
					}
				} else {
					throw new Exception(
							"Unique parameter doesn't match the other parameter from array! ");
				}
			}
		}
		
		return null;

	}

	/**
	 * Converts Object into java.util.Date
	 * 
	 * @param obj
	 * @return 
	 */
	private Date getDate(Object obj) {
		Calendar aa = Calendar.getInstance();
		aa.setTime((java.util.Date) obj);
		Date ret = new Date(aa.getTimeInMillis());
		return ret;
	}

	/**
	 * Fill the map with parameters values
	 *
	 */
	private void fillTheMap() {
		for (int i = 0; i < parameters.length; i++) {
			parameterMap.put(parameters[i].the_formal_name,
					parameters[i].the_value);
		}
	}

	private Method findMethod(String className, String methodName)
			throws SecurityException, NoSuchMethodException,
			ClassNotFoundException {
		return mth = Class.forName(className).getMethod(methodName, null);
	}

	/**
	 * Read extended attributes
	 */
	protected ExtendedAttributes readParamsFromExtAttributes(String extAttribs)
			throws Exception {
		ExtendedAttributes eas = super.readParamsFromExtAttributes(extAttribs);
		ExtendedAttribute ea = eas
				.getFirstExtendedAttributeForName(StorageTool.DATA_TABLE_CLASS_EXT_ATTR_NAME);
		if (ea != null) {
			fullpackageName = ea.getVValue();
		}

		return eas;
	}

}
