package org.enhydra.wlh.business;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMAttribute;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmc.wapi.WMSessionHandle;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.client.wfservice.XPDLBrowser;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.wlh.spec.SharkParamConsts;
import org.w3c.dom.Node;

public class SharkUtils {

   private static String[] allowedDTypes = {
         "java.lang.String",
         "java.lang.Boolean",
         "java.lang.Double",
         "java.lang.Long",
         "java.util.Date",
         "org.w3c.dom.Node"
   };

   public static final List allowedDataTypes = Arrays.asList(allowedDTypes);

   public static void completeActivity(String username, String procId, String actId)
      throws Exception {

      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");
      sc.connect(wmci);
      WfActivity act = sc.getActivity(procId, actId);
      if (act.state().equals(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED)) {
         act.change_state(SharkConstants.STATE_OPEN_RUNNING);
      }
      if (!act.state().equals(SharkConstants.STATE_CLOSED_COMPLETED))
         act.complete();
      sc.disconnect();

   }

   public static void updateVariablesDescAndPriority(String username,
                                                     String procId,
                                                     String actId,
                                                     Map map,
                                                     String proDesc,
                                                     int proPrio) throws Exception {

      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");
      sc.connect(wmci);
      WfActivity act = sc.getActivity(procId, actId);

      act.set_result(map);

      act.container().set_description(proDesc);
      act.container().set_priority((short) proPrio);

      sc.disconnect();
   }

   public static void acceptedActivity(String username,
                                       String procId,
                                       String actId,
                                       String accepted) throws Exception {
      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");
      sc.connect(wmci);
      WfActivity act = sc.getActivity(procId, actId);

      if (accepted.equals("false")
          && !act.state().equals(SharkConstants.STATE_OPEN_RUNNING))
         act.change_state(SharkConstants.STATE_OPEN_RUNNING);
      else if (!accepted.equals("false")
               && !act.state().equals(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED))
         act.change_state(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED);
      sc.disconnect();
   }

   // if checkFirstActivity is true and the first (manual) activity in the
   // process belongs to the user that started the process, activity's Id is
   // returned
   public static WfActivity createProcess(String username,
                                          String manName,
                                          boolean checkFirstActivity) throws Exception {

      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");
      sc.connect(wmci);
      WfProcessMgr pMan = sc.getProcessMgr(manName);
      WfProcess proc = pMan.create_process(null);
      proc.start();
      sc.disconnect();

      if (checkFirstActivity) {
         return findNextActivity(proc, username);
      }
      return null;
   }

   public static WfActivity findNextActivity(WfProcess proc, String username)
      throws Exception {
      WfActivity[] acts = proc.get_sequence_step(0);
      WfActivity nextAct = null;
      for (int i = 0; i < acts.length; i++) {
         if (acts[i].state().equals(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED)) {
            WfAssignment[] ass = acts[i].get_sequence_assignment(0);
            if (ass.length == 0) {
               nextAct = acts[i];
               break;
            }
            for (int j = 0; j < ass.length; j++) {
               if (ass[j].assignee().resource_key().equals(username)) {
                  nextAct = acts[i];
               }
            }
            if (nextAct != null) {
               break;
            }
         }
      }
      return nextAct;
   }

   public static Map filterActivityVariables(Map proVars,
                                             Map actVars,
                                             Map typeList,
                                             String[][] extAttribs) throws Exception {

      Map fetchSources = new HashMap();
      for (int i = 0; i < extAttribs.length; i++) {
         String eaName = extAttribs[i][0];

         if (eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_UPDATE)
             || eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_VIEW)
             || eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_FETCH)) {
            String variableId = extAttribs[i][1];
            String fetchVarId = null;
            int indOfSC = variableId.indexOf(";");
            if (indOfSC > 0) {
               fetchVarId = variableId.substring(indOfSC + 1);
               variableId = variableId.substring(0, indOfSC);
            }
            if (proVars.containsKey(variableId)) {

               actVars.put(variableId, proVars.get(variableId));

               if (eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_UPDATE)) {
                  typeList.put(variableId, new Integer(1));
               } else if (eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_VIEW)) {
                  typeList.put(variableId, new Integer(0));
               } else if (eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_FETCH)) {
                  typeList.put(variableId, new Integer(2));
                  typeList.put(fetchVarId, new Integer(-1));
                  fetchSources.put(variableId, fetchVarId);
                  actVars.put(fetchVarId, proVars.get(fetchVarId));
               }
            }
         }
      }

      return fetchSources;
   }

   public static boolean isVariableReadOnly(String varId, String[][] extAttribs)
      throws Exception {

      boolean retVal = false;
      for (int i = 0; i < extAttribs.length; i++) {
         String eaName = extAttribs[i][1];
         if (eaName.equals(varId)) {

            String readOnly = extAttribs[i][0];

            if (readOnly.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_VIEW)) {
               return true;
            }
            return false;
         }
      }

      return retVal;
   }

   public static boolean doesXFormExist(String[][] extAttribs) throws Exception {
      for (int i = 0; i < extAttribs.length; i++) {
         String eaName = extAttribs[i][0];

         if (eaName.equals(SharkParamConsts.XFORMS_XMLC_CLASS)) {

            return true;
         }
      }

      return false;

   }

   /**
    * This methode take map of variables and map of context_signature and return list of
    * VariableData.
    * 
    * @param varMap map of varibles
    * @param cs context signature for process
    * @return list of varibles
    * @throws Exception
    */
   public static List classifyVariables(Map varMap, Map cs) throws Exception {
      Iterator iter = varMap.keySet().iterator();
      List lst = new ArrayList();

      while (iter.hasNext()) {

         Object key = iter.next();
         Object obj = cs.get(key);
         if (obj == null) {
            Object var = varMap.get(key);
            if (var != null) {
               obj = var.getClass().getName();
               if (var instanceof Node) {
                  obj = "org.w3c.dom.Node";
               }
            }
         }

         if (obj != null) {

            if (allowedDataTypes.contains(obj)) {

               VariableData varData = new VariableData();

               varData.setId(key.toString());
               varData.setValue(varMap.get(key));
               varData.setType((String) obj);
               lst.add(varData);
            }
         }
      }

      return lst;

   }

   public static String[][] getExtAttribNVPairs(WMSessionHandle sh, WMEntity ent)
      throws Exception {
      XPDLBrowser xpdlb = Shark.getInstance().getXPDLBrowser();
      WMFilter filter = new WMFilter("Type", WMFilter.EQ, "ExtendedAttribute");

      WMEntity[] enti = xpdlb.listEntities(sh, ent, filter, true).getArray();
      String[][] eaNVP = new String[enti.length][2];

      for (int i = 0; i < enti.length; i++) {
         WMEntity ea = enti[i];
         WMAttribute[] attrs = xpdlb.listAttributes(sh, ea, null, true).getArray();
         eaNVP[i][0] = attrs[0].getValue().toString();
         eaNVP[i][1] = attrs[1].getValue().toString();
      }

      return eaNVP;
   }

   /**
    * Completes activity and returns true if process is not finished yet.
    */
   public static boolean continueProcess(String username,
                                         String procId,
                                         String actId,
                                         Map map) throws Exception {
      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");
      sc.connect(wmci);
      WfActivity act = sc.getActivity(procId, actId);
      WfProcess pro = act.container();

      act.set_result(map);

      if (act.state().equals(SharkConstants.STATE_OPEN_NOT_RUNNING_NOT_STARTED)) {
         act.change_state(SharkConstants.STATE_OPEN_RUNNING);
      }
      act.complete();

      boolean running = true;
      if (!pro.state().equals(SharkConstants.STATE_OPEN_RUNNING)) {
         running = false;
      }

      sc.disconnect();

      return running;

   }

   public static boolean isContextUpdated(String username, String procId, String actId)
      throws Exception {

      boolean updated = true;
      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");

      sc.connect(wmci);

      WfActivity act = sc.getActivity(procId, actId);
      Map res = act.result();
      if (res.size() == 0) {
         AdminMisc am = Shark.getInstance().getAdminMisc();
         WMEntity varEnt = am.getActivityDefinitionInfo(sc.getSessionHandle(),
                                                        procId,
                                                        actId);

         String[][] extAttribs = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(),
                                                                varEnt);

         for (int i = 0; i < extAttribs.length; i++) {
            String eaName = extAttribs[i][0];
            if (eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_UPDATE)
                || eaName.equals(SharkParamConsts.EXT_ATTRIB_VAR_TO_PROCESS_FETCH)) {
               updated = false;
               break;
            }
         }
      }

      return updated;
   }

   /**
    * Check activity extends attributes and retun type of output if return 0 - output will
    * be xhtml with xforms 2 - output will be generic xhtml-xforms page
    */
   public static int getOutputType(String username, String procId, String actId)
      throws Exception {
      SharkConnection sc = Shark.getInstance().getSharkConnection();
      WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");

      sc.connect(wmci);

      AdminMisc am = Shark.getInstance().getAdminMisc();
      WMEntity varEnt = am.getActivityDefinitionInfo(sc.getSessionHandle(), procId, actId);

      String[][] extAttribs = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(),
                                                             varEnt);

      for (int i = 0; i < extAttribs.length; i++) {
         String eaName = extAttribs[i][0];
         if (eaName.equals(SharkParamConsts.XFORMS_XMLC_CLASS)) {
            return 0;
         }
      }
      return 2;
   }

}
