package org.enhydra.wlh.business;

import java.sql.Date;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlObject;
import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfActivityIterator;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.PackageAdministration;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.common.ActivityFilterBuilder;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.shark.xpil.XPILDataInstance;
import org.enhydra.shark.xpil.XPILWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPDLDataFieldDocument.DataField;
import org.enhydra.shark.xpil.XPILActivityInstancesDocument.ActivityInstances;
import org.enhydra.shark.xpil.XPILAssignmentInstanceDocument.AssignmentInstance;
import org.enhydra.shark.xpil.XPILAssignmentInstancesDocument.AssignmentInstances;
import org.enhydra.shark.xpil.XPILBooleanDataInstanceDocument.BooleanDataInstance;
import org.enhydra.shark.xpil.XPILDataInstancesDocument.DataInstances;
import org.enhydra.shark.xpil.XPILDateDataInstanceDocument.DateDataInstance;
import org.enhydra.shark.xpil.XPILDoubleDataInstanceDocument.DoubleDataInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILInstanceExtendedAttributeDocument.InstanceExtendedAttribute;
import org.enhydra.shark.xpil.XPILInstanceExtendedAttributesDocument.InstanceExtendedAttributes;
import org.enhydra.shark.xpil.XPILLongDataInstanceDocument.LongDataInstance;
import org.enhydra.shark.xpil.XPILManualActivityInstanceDocument.ManualActivityInstance;
import org.enhydra.shark.xpil.XPILSchemaDataInstanceDocument.SchemaDataInstance;
import org.enhydra.shark.xpil.XPILStringDataInstanceDocument.StringDataInstance;
import org.enhydra.shark.xpil.XPILWorkflowProcessFactoryInstanceDocument.WorkflowProcessFactoryInstance;
import org.enhydra.wlh.spec.SharkParamConsts;
import org.w3c.dom.Node;

import com.lutris.util.Config;

public class SharkXPILBindingUtils {

   public static WorkflowProcessFactoryInstance[] getProcessFactories(SharkConnection sc,
                                                                      Config config,
                                                                      String propertyPrefix)
      throws Exception {

      WfProcessMgr[] wfRs = sc.get_sequence_processmgr(0);

      List mgrsToDisplay=new ArrayList();
      boolean showOnlyLastVers = config.getBoolean("WorkListHandler."
                                          + propertyPrefix
                                          + ".showlastfactoryversiononly", false);

      if (showOnlyLastVers) {
         Map pkgsToCurVers=new HashMap();
         AdminMisc am=Shark.getInstance().getAdminMisc();
         PackageAdministration pa=Shark.getInstance().getPackageAdministration();
         for (int i=0; i<wfRs.length; i++) {
            WfProcessMgr mgr=wfRs[i];
            WMEntity pDefEnt=am.getProcessDefinitionInfoByUniqueProcessDefinitionName(sc.getSessionHandle(), mgr.name());
            String pkgId=pDefEnt.getPkgId();
            String curVer=(String)pkgsToCurVers.get(pkgId);
            if (curVer==null) {
               curVer=pa.getCurrentPackageVersion(sc.getSessionHandle(), pkgId);
               pkgsToCurVers.put(pkgId, curVer);
            }
            if (pDefEnt.getPkgVer().equals(curVer)) {
               mgrsToDisplay.add(mgr);
            }
         }
      } else {
         mgrsToDisplay.addAll(Arrays.asList(wfRs));
      }
      ArrayList al=new ArrayList();
     
      
      for (int i = 0; i < mgrsToDisplay.size(); i++) {
         
         WorkflowProcessFactoryInstance wpfi = getProcessFactory(sc, config, propertyPrefix, (WfProcessMgr)mgrsToDisplay.get(i));
         if((wpfi.getState().toString()).equalsIgnoreCase("enabled")){
           al.add(wpfi);
         }
      }
      WorkflowProcessFactoryInstance[] factoryInstances = new WorkflowProcessFactoryInstance[al.size()];

      al.toArray(factoryInstances);

      return factoryInstances;

   }

   public static WorkflowProcessFactoryInstance getProcessFactory(SharkConnection sConn,
                                                                  Config config,
                                                                  String propertyPrefix,
                                                                  WfProcessMgr wfPm)
      throws Exception {

      WorkflowProcessFactoryInstance wpfi = WorkflowProcessFactoryInstance.Factory.newInstance();

      wpfi.setId(wfPm.name());
      wpfi.setState(WorkflowProcessFactoryInstance.State.Enum.forInt(1+wfPm.process_mgr_state()
         .value()));

      AdminMisc am = Shark.getInstance().getAdminMisc();

      WMEntity wmEnty = am.getProcessDefinitionInfoByUniqueProcessDefinitionName(sConn.getSessionHandle(),
                                                                                 wfPm.name());

      wpfi.addNewWorkflowProcess();

      wpfi.getWorkflowProcess().setName(wmEnty.getName());
      wpfi.getWorkflowProcess().setId(wmEnty.getId());

      return wpfi;
   }

   public static void fillProcess(SharkConnection sc,
                                  Config config,
                                  String propertyPrefix,
                                  WfProcess process,
                                  XPILWorkflowProcessInstance mProcess) throws Exception {

      String procId = process.key();
      mProcess.setId(procId);
      mProcess.setName(process.name());
      mProcess.setState(XPILWorkflowProcessInstance.State.Enum.forString(process.state()));
      mProcess.setInstanceDescription(process.description());
      mProcess.setInstancePriority(process.priority());

      Calendar c = Calendar.getInstance();
      AdminMisc am = Shark.getInstance().getAdminMisc();

      Date date = new Date(am.getProcessCreatedTime(sc.getSessionHandle(), procId));
      c.setTime(date);
      mProcess.setCreated(c);

      date = new Date(am.getProcessStartedTime(sc.getSessionHandle(), procId));
      c.setTime(date);
      try {
         mProcess.setStarted(c);
      } catch (Exception ex) {
      }

      date = new Date(am.getProcessFinishTime(sc.getSessionHandle(), procId));
      c.setTime(date);
      try {
         mProcess.setFinished(c);
      } catch (Exception ex) {
      }

      mProcess.setRequesterUsername(am.getProcessRequesterUsername(sc.getSessionHandle(),
                                                                   process.key()));

      WMEntity procEnt = am.getProcessDefinitionInfo(sc.getSessionHandle(), procId);
      mProcess.setDefinitionId(procEnt.getId());

      boolean fillActivityInfo = config.getBoolean("WorkListHandler."
                                                         + propertyPrefix
                                                         + ".fill.xpil.runningactivities",
                                                   false);
      if (fillActivityInfo) {
         ActivityInstances ais = mProcess.addNewActivityInstances();

         ais.setManualActivityInstanceArray(getRunningManualActivities(sc,
                                                                       config,
                                                                       propertyPrefix,
                                                                       process));
      }

      boolean fillVar = config.getBoolean("WorkListHandler."
                                          + propertyPrefix
                                          + ".fill.xpil.processvariables", false);
      if (fillVar) {
         mProcess.addNewDataInstances();

         DataInstances dataIns = mProcess.getDataInstances();

         fillVariables(sc,
                       config,
                       propertyPrefix,
                       dataIns,
                       process,
                       process.process_context());
      }

   }

   public static ManualActivityInstance[] getRunningManualActivities(SharkConnection sc,
                                                                     Config config,
                                                                     String propertyPrefix,
                                                                     WfProcess process)
      throws Exception {

      WfActivityIterator ai = sc.get_iterator_activity();
      ActivityFilterBuilder fb = Shark.getInstance().getActivityFilterBuilder();
      WMFilter filter = fb.addProcessIdEquals(sc.getSessionHandle(), process.key());
      filter = fb.and(sc.getSessionHandle(),
                      filter,
                      fb.addStateStartsWith(sc.getSessionHandle(),
                                            SharkConstants.STATEPREFIX_OPEN));
      ai.set_query_expression(fb.toIteratorExpression(sc.getSessionHandle(), filter));
      WfActivity[] acts = ai.get_next_n_sequence(0);

      ManualActivityInstance[] mActs = new ManualActivityInstance[acts.length];
      for (int i = 0; i < acts.length; i++) {

         WfActivity act = acts[i];

         mActs[i] = getManualActivity(sc, config, propertyPrefix, act);

      }

      return mActs;
   }

   public static ManualActivityInstance getManualActivity(SharkConnection sc,
                                                          Config config,
                                                          String propertyPrefix,
                                                          WfActivity act)
      throws Exception {

      String procId = act.container().key();
      AdminMisc am = Shark.getInstance().getAdminMisc();
      ManualActivityInstance mAct = ManualActivityInstance.Factory.newInstance();

      mAct.setId(act.key());
      mAct.setName(act.name());
      mAct.setState(XPILWorkflowProcessInstance.State.Enum.forString(act.state()));
      mAct.setInstanceDescription(act.description());
      mAct.setInstancePriority(act.priority());

      Calendar c = Calendar.getInstance();

      Date date = new Date(am.getActivityCreatedTime(sc.getSessionHandle(),
                                                     procId,
                                                     act.key()));

      c.setTime(date);

      mAct.setCreated(c);

      try {
         date = new Date(am.getActivityStartedTime(sc.getSessionHandle(),
                                                   procId,
                                                   act.key()));
         c.setTime(date);
         mAct.setStarted(c);
      } catch (Exception ex) {
      }

      try {
         date = new Date(am.getActivityFinishTime(sc.getSessionHandle(),
                                                  procId,
                                                  act.key()));
         c.setTime(date);
         mAct.setFinished(c);
      } catch (Exception ex) {
      }

      boolean fillVar = config.getBoolean("WorkListHandler."
                                          + propertyPrefix
                                          + ".fill.xpil.activityvariables", false);
      if (fillVar) {
         mAct.addNewDataInstances();
         DataInstances dataIns = mAct.getDataInstances();

         SharkXPILBindingUtils.fillVariables(sc,
                                             config,
                                             propertyPrefix,
                                             dataIns,
                                             act.container(),
                                             act.process_context());
      }
      boolean fillAss = config.getBoolean("WorkListHandler."
                                          + propertyPrefix
                                          + ".fill.xpil.activityassignments", true);
      if (fillAss) {
         mAct.addNewAssignmentInstances();

         AssignmentInstances aIns = mAct.getAssignmentInstances();

         WfAssignment[] ass = act.get_sequence_assignment(0);
         AssignmentInstance[] assArray = new AssignmentInstance[ass.length];

         for (int j = 0; j < ass.length; j++) {

            WfAssignment as = ass[j];

            AssignmentInstance assIns = AssignmentInstance.Factory.newInstance();
            assIns.setIsAccepted(as.get_accepted_status());
            assIns.setUsername(as.assignee().resource_key());

            assArray[j] = assIns;

         }
         aIns.setAssignmentInstanceArray(assArray);
      }
      return mAct;
   }

   public static void fillVariables(SharkConnection sc,
                                    Config config,
                                    String propertyPrefix,
                                    DataInstances ins,
                                    WfProcess pro,
                                    Map varMap) throws Exception {

      WfProcessMgr mgr = pro.manager();
      Map cs = mgr.context_signature();

      List l = SharkUtils.classifyVariables(varMap, cs);

      Iterator it = l.iterator();

      
      AdminMisc am=Shark.getInstance().getAdminMisc();
      String proId=pro.key();
      
      while (it.hasNext()) {
         VariableData vData = (VariableData) it.next();
         
         String key = vData.getType();
         XPILDataInstance di=null;
         if (key.equals("java.lang.String")) {
            di = ins.addNewStringDataInstance();
            ((StringDataInstance)di).setValue(((String) vData.getValue()));
         } else if (key.equals("java.lang.Double")) {
            di = ins.addNewDoubleDataInstance();;
            ((DoubleDataInstance)di).setValue(((Double) vData.getValue()).doubleValue());
         } else if (key.equals("java.lang.Long")) {
            di = ins.addNewLongDataInstance();
            ((LongDataInstance)di).setValue(((Long) vData.getValue()).longValue());
         } else if (key.equals("java.util.Date")) {
            di = ins.addNewDateDataInstance();
            java.util.Date date = (java.util.Date) vData.getValue();
            Calendar now = Calendar.getInstance();
            now.setTime(date);
            ((DateDataInstance)di).setValue(now);
         } else if (key.equals("java.lang.Boolean")) {
            di = ins.addNewBooleanDataInstance();
            ((BooleanDataInstance)di).setValue(((Boolean) vData.getValue()).booleanValue());
         } else if (key.equals("org.w3c.dom.Node")) {
            di = ins.addNewSchemaDataInstance();
            XmlObject xmlO = XmlObject.Factory.parse((Node) vData.getValue());
            ((SchemaDataInstance)di).setValue(xmlO);
         }
         if (di!=null) {
            WMEntity vEnt = am.getVariableDefinitionInfo(sc.getSessionHandle(),
                                                         proId,
                                                         vData.getId());
            
            di.setId(vData.getId());
            DataField df = di.addNewDataField();
            df.setId(vEnt.getId());
            df.setName(vEnt.getName());
            InstanceExtendedAttributes ieas = di.addNewInstanceExtendedAttributes();
            InstanceExtendedAttribute iea = null;
            if (vData.getType().equals("java.lang.String")) {
               String[][] vEAs = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(), vEnt);
               for (int i=0; i<vEAs.length; i++) {
                  if (vEAs[i][0].equals(SharkParamConsts.EXT_ATTRIB_VAR_RENDERING_HINT)) {
                     if (vEAs[i][1].equals("Size=L")) {
                        iea = ieas.addNewInstanceExtendedAttribute();
                        iea.setName(SharkParamConsts.EXT_ATTRIB_VAR_RENDERING_HINT);
                        iea.setValue("Size=L");                        
                     }
                     break;
                  }
               }
            }
            
         }
      }

   }

   public static XPILDataInstance fillVariable(SharkConnection sc,
                                               Config config,
                                               String propertyPrefix,
                                               ExtendedWorkflowFacilityInstance ins,
                                               DataInstances dins,
                                               VariableData vData) throws Exception {

      XPILDataInstance var = null;
      String vType = vData.getType();
      if (vType.equals("java.lang.String")) {
         StringDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewStringDataInstance();
         } else {
            vd = dins.addNewStringDataInstance();
         }
         vd.setId(vData.getId());
         vd.setValue(((String) vData.getValue()));
         var = vd;
      } else if (vType.equals("java.lang.Double")) {

         DoubleDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewDoubleDataInstance();
         } else {
            vd = dins.addNewDoubleDataInstance();
         }
         vd.setId(vData.getId());
         vd.setValue(((Double) vData.getValue()).doubleValue());
         var = vd;

      } else if (vType.equals("java.lang.Long")) {

         LongDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewLongDataInstance();
         } else {
            vd = dins.addNewLongDataInstance();
         }
         vd.setId(vData.getId());
         vd.setValue(((Long) vData.getValue()).longValue());
         var = vd;
      } else if (vType.equals("java.util.Date")) {

         DateDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewDateDataInstance();
         } else {
            vd = dins.addNewDateDataInstance();
         }
         vd.setId(vData.getId());
         java.util.Date date = (java.util.Date) vData.getValue();
         XmlDate a = XmlDate.Factory.newInstance();
         SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd");
         a.setStringValue(f.format(date));
         vd.xsetValue(a);
         var = vd;
      } else if (vType.equals("java.lang.Boolean")) {

         BooleanDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewBooleanDataInstance();
         } else {
            vd = dins.addNewBooleanDataInstance();
         }
         vd.setId(vData.getId());
         vd.setValue(((Boolean) vData.getValue()).booleanValue());

         var = vd;

      } else if (vType.equals("org.w3c.dom.Node")) {

         SchemaDataInstance vd = null;
         if (ins != null) {
            vd = ins.addNewSchemaDataInstance();
         } else {
            vd = dins.addNewSchemaDataInstance();
         }
         vd.setId(vData.getId());
         XmlObject xmlO = XmlObject.Factory.parse((Node) vData.getValue());
         vd.setValue(xmlO);

         var = vd;

      }
      return var;

   }

}
