package org.enhydra.wlh.business;

import java.sql.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.utilities.SequencedHashMap;
import org.enhydra.shark.xpil.XPILDataInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPDLDataFieldDocument.DataField;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILHeaderDocument.Header;
import org.enhydra.shark.xpil.XPILInstanceExtendedAttributeDocument.InstanceExtendedAttribute;
import org.enhydra.shark.xpil.XPILInstanceExtendedAttributesDocument.InstanceExtendedAttributes;
import org.enhydra.shark.xpil.XPILManualActivityInstanceDocument.ManualActivityInstance;
import org.enhydra.wlh.spec.SharkParamConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;

import com.lutris.util.Config;

public class XMLActivityBuilder extends XMLBuilderImpl {

   protected String proId = null;

   protected String actId = null;

   public XMLActivityBuilder(String username,
                             String proID,
                             String actID,
                             Config congfig,
                             String startElement,                             
                             String sortCriterion, 
                             String sortAsc) {
      this.username = username;
      this.proId = proID;
      this.actId = actID;
      this.config = congfig;
      this.from_element = startElement;
      this.to_element = "9";
      this.total_number = "0";
      this.sortCriterion = sortCriterion;
      this.sortAsc = sortAsc;
   }

   public XPILExtendedWorkflowFacilityInstanceDocument fillXMLElements(SharkConnection sc)
      throws Exception {

      XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.newInstance();

      tws.addNewExtendedWorkflowFacilityInstance();

      ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();
      ins.addNewHeader();
      Header header = tws.getExtendedWorkflowFacilityInstance().getHeader();
      header.setXPILVersion("1.0");
      header.setXPILVendor("Shark");

      GregorianCalendar gC = new GregorianCalendar();
      gC.setTime(new Date(System.currentTimeMillis()));

      header.setCreationTime(gC);

      header.setInstanceDescription("Shark's activity details");

      WfActivity act = sc.getActivity(proId, actId);
      WfProcess pro = act.container();

      XPILWorkflowProcessInstance p = null;
      if (pro.requester() instanceof WfActivity) {
         p = ins.addNewSubWorkflowProcessInstance();
      } else {
         p = ins.addNewMainWorkflowProcessInstance();
      }
      p.setId(proId);
      p.setName(pro.name());
      p.setInstanceDescription(pro.description());
      p.setInstancePriority(pro.priority());

      ManualActivityInstance mAct = ins.addNewManualActivityInstance();
      mAct.setId(act.key());
      mAct.setName(act.name());
      mAct.setInstanceDescription(act.description());
      mAct.setInstancePriority(act.priority());

      Map varMap = act.process_context();
      Map actMap = new SequencedHashMap();

      // keys are variable ids and values are integers that represent what type of
      // extended attribute to store: -1,0,1,2
      Map typeList = new HashMap();

      AdminMisc am = Shark.getInstance().getAdminMisc();
      WMEntity actEnt = am.getActivityDefinitionInfo(sc.getSessionHandle(),
                                                     pro.key(),
                                                     actId);

      String[][] ext = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(), actEnt);
      WfProcessMgr mgr = pro.manager();
      Map cs = mgr.context_signature();
      if (SharkUtils.doesXFormExist(ext)) {

         // List l = SharkUtils.classifyVariables(varMap, cs);

      } else {
         Map fetchSources = SharkUtils.filterActivityVariables(varMap,
                                                               actMap,
                                                               typeList,
                                                               ext);
         List l = SharkUtils.classifyVariables(actMap, cs); // returns the list of
                                                            // VariableData objects

         Iterator it = l.iterator();

         while (it.hasNext()) { // there are more VariableData objects to check
            VariableData vData = (VariableData) it.next();
            XPILDataInstance di = SharkXPILBindingUtils.fillVariable(sc,
                                                                     config,
                                                                     ParamConsts.PROPERTY_PREFIX_ACTIVITYDETAILS,
                                                                     ins,
                                                                     null,
                                                                     vData);

            WMEntity vEnt = am.getVariableDefinitionInfo(sc.getSessionHandle(),
                                                         proId,
                                                         vData.getId());
            DataField df = di.addNewDataField();
            df.setId(vEnt.getId());
            df.setName(vEnt.getName());
            InstanceExtendedAttributes ieas = di.addNewInstanceExtendedAttributes();
            InstanceExtendedAttribute iea = ieas.addNewInstanceExtendedAttribute();
            iea.setName("how_to_handle_marker");
            iea.setValue(String.valueOf(((Integer) typeList.get(vData.getId())).intValue()));
            int handleType=((Integer) typeList.get(vData.getId())).intValue();
            if (handleType==1 && vData.getType().equals("java.lang.String")) {
               String[][] vEAs = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(), vEnt);
               for (int i=0; i<vEAs.length; i++) {
                  if (vEAs[i][0].equals(SharkParamConsts.EXT_ATTRIB_VAR_RENDERING_HINT)) {
                     if (vEAs[i][1].equals("Size=L")) {
                        iea = ieas.addNewInstanceExtendedAttribute();
                        iea.setName(SharkParamConsts.EXT_ATTRIB_VAR_RENDERING_HINT);
                        iea.setValue("Size=L");                        
                     }
                     break;
                  }
               }
            }
            if (handleType == 2) {
               iea = ieas.addNewInstanceExtendedAttribute();
               iea.setName("fetch_marker");
               iea.setValue((String) fetchSources.get(vData.getId()));
            }
         }
      }

//      System.out.println("XMLActivityBuilder = " + tws.toString());

      return tws;
   }

}
