package org.enhydra.wlh.business;

import java.sql.Date;
import java.util.GregorianCalendar;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessIterator;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.common.ProcessFilterBuilder;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.shark.utilities.MiscUtilities;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILHeaderDocument.Header;
import org.enhydra.wlh.spec.utils.ParamConsts;

import com.lutris.util.Config;

public class XMLProcessInstanceBuilder extends XMLBuilderImpl {
   
   public XMLProcessInstanceBuilder(String username, Config config, String startElement, String sortCriterion, String sortAsc) {
      this.username = username;
      this.config = config;
      this.from_element = startElement;
      this.sortCriterion = sortCriterion;
      this.sortAsc = sortAsc;
   }

   protected XPILExtendedWorkflowFacilityInstanceDocument fillXMLElements(SharkConnection sc)
      throws Exception {
      XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.newInstance();

      tws.addNewExtendedWorkflowFacilityInstance();

      ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();
      ins.addNewHeader();
      Header header = tws.getExtendedWorkflowFacilityInstance().getHeader();
      header.setXPILVersion("1.0");
      header.setXPILVendor("Shark");

      GregorianCalendar gC = new GregorianCalendar();
      gC.setTime(new Date(System.currentTimeMillis()));

      header.setCreationTime(gC);

      header.setInstanceDescription("Shark's process instance list");

      int itemsPerPage = config.getInt("WorkListHandler."
                                       + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                       + ".max.rows.per.page", 5);

      int startAt = Integer.parseInt(from_element != null ? from_element : "0");

      boolean myProcessesOnly = config.getBoolean("WorkListHandler."
                                                        + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                                        + ".fill.xpil.onlymyprocesses",
                                                  false);
      String superusers[] = config.getStrings("WorkListHandler."
                                              + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                              + ".superusers");
  
      
      boolean isSuperUser = false;
      if (superusers != null) {
         for (int i = 0; i < superusers.length; i++) {
            if (superusers[i].equals(username)) {
               isSuperUser = true;
               break;
            }
         }
      }
      if (isSuperUser) {
         myProcessesOnly = false;
      }
      boolean openProcessesOnly = config.getBoolean("WorkListHandler."
                                                          + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                                          + ".fill.xpil.onlyopenprocesses",
                                                    false);

      WfProcessIterator pit = sc.get_iterator_process();
      ProcessFilterBuilder fb = Shark.getInstance().getProcessFilterBuilder();
      WMFilter filter = fb.createEmptyFilter(sc.getSessionHandle());
      if (myProcessesOnly) {
         filter = fb.and(sc.getSessionHandle(),
                         filter,
                         fb.addRequesterUsernameEquals(sc.getSessionHandle(), username));
      }
      if (openProcessesOnly) {
         filter = fb.and(sc.getSessionHandle(),
                         filter,
                         fb.addStateStartsWith(sc.getSessionHandle(),
                                               SharkConstants.STATEPREFIX_OPEN));
      }
     
      String exp = fb.toIteratorExpression(sc.getSessionHandle(), filter);
      pit.set_query_expression(exp);
      int hm = pit.how_many();
      if (startAt > hm) {
         startAt = (hm / itemsPerPage) * itemsPerPage;
         from_element = Integer.toString(startAt);
      } else if (startAt == hm && hm > 0) {
         startAt = startAt - itemsPerPage;
         from_element = Integer.toString(startAt);
      }
      from_element = Integer.toString(startAt);
      to_element = Integer.toString(startAt + itemsPerPage - 1);
      total_number = Integer.toString(hm);

      filter = fb.setLimit(sc.getSessionHandle(), filter, itemsPerPage);
      filter = fb.setStartPosition(sc.getSessionHandle(), filter, startAt);
     
      if(this.sortCriterion == null || this.sortCriterion.equals("")) { //search as declared in config file
         String orderByList = config.getString("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                               + ".fill.xpil.orderbylist", "");
         String[] obstrs = MiscUtilities.tokenize(orderByList, ";");
         if (obstrs.length > 0) {
            for (int i = 0; i < obstrs.length; i++) {
               String obs = obstrs[i];
               String[] sortinfo = MiscUtilities.tokenize(obs, ",");
               if (sortinfo != null && sortinfo.length == 2) {
                  String orderBy = sortinfo[0];
                  boolean asc = new Boolean(sortinfo[1]).booleanValue();
                  if (orderBy.equals("state")) {
                     fb.setOrderByState(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("createdtime")) {
                     fb.setOrderByCreatedTime(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("priority")) {
                     fb.setOrderByPriority(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("requester")) {
                     fb.setOrderByResourceRequesterId(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("name")) {
                     fb.setOrderByName(sc.getSessionHandle(), filter, asc);
                  }
               }
            }
         }
      }
      // user has chosen search criterion
      // can search by state, createdTime, priority, requester, name 
      else {
         if(this.sortCriterion.equalsIgnoreCase("state")) {
            fb.setOrderByState(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue() );
         }
         else if(this.sortCriterion.equalsIgnoreCase("createdtime")) {
            fb.setOrderByCreatedTime(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue());
         }
         else if(this.sortCriterion.equalsIgnoreCase("priority")) {
            fb.setOrderByPriority(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue());
         }
         else if(this.sortCriterion.equalsIgnoreCase("requester")) {
            fb.setOrderByResourceRequesterId(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue());
         }
         else if(this.sortCriterion.equalsIgnoreCase("name")) {
            fb.setOrderByName(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue());
         }
      }
      

      exp = fb.toIteratorExpression(sc.getSessionHandle(), filter);
      pit.set_query_expression(exp);
      WfProcess[] procs = pit.get_next_n_sequence(0);

      for (int i = 0; i < procs.length; i++) {
         WfProcess pro = procs[i];
         XPILWorkflowProcessInstance p = null;
         if (pro.requester() instanceof WfActivity) {
            p = ins.addNewSubWorkflowProcessInstance();
         } else {
            p = ins.addNewMainWorkflowProcessInstance();
         }
         SharkXPILBindingUtils.fillProcess(sc,
                                           config,
                                           ParamConsts.PROPERTY_PREFIX_PROCESSLIST,
                                           pro,
                                           p);
      }

      boolean pManagerList = config.getBoolean("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                                               + ".fill.xpil.processfactories", true);
      if (pManagerList) {
         ins.setWorkflowProcessFactoryInstanceArray(SharkXPILBindingUtils.getProcessFactories(sc,
                                                                                              config,
                                                                                              ParamConsts.PROPERTY_PREFIX_PROCESSLIST));
      }

      return tws;

   }

}
