package org.enhydra.wlh.business;

import java.sql.Date;
import java.util.GregorianCalendar;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.client.wfservice.WfProcessMgrIterator;
import org.enhydra.shark.api.common.ProcessMgrFilterBuilder;
import org.enhydra.shark.utilities.MiscUtilities;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILHeaderDocument.Header;
import org.enhydra.shark.xpil.XPILPackageInstanceDocument.PackageInstance;
import org.enhydra.shark.xpil.XPILPackageInstancesDocument.PackageInstances;
import org.enhydra.shark.xpil.XPILWorkflowProcessFactoryInstanceDocument.WorkflowProcessFactoryInstance;
import org.enhydra.wlh.spec.utils.ParamConsts;

import com.lutris.util.Config;

public class XMLProcessInstantiationBuilder extends XMLBuilderImpl {

   private WfProcessMgr currentMgr = null;

   public XMLProcessInstantiationBuilder(String username,
                                         Config config,
                                         String startElement,
                                         String sortCriterion,
                                         String sortAsc) {

      this.username = username;
      this.config = config;
      this.from_element = startElement;
      this.sortCriterion = sortCriterion;
      this.sortAsc = sortAsc;

   }

   protected XPILExtendedWorkflowFacilityInstanceDocument fillXMLElements(SharkConnection sc)
      throws Exception {
      XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.newInstance();

      tws.addNewExtendedWorkflowFacilityInstance();

      ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();
      ins.addNewHeader();
      Header header = tws.getExtendedWorkflowFacilityInstance().getHeader();
      header.setXPILVersion("1.0");
      header.setXPILVendor("Shark");

      GregorianCalendar gC = new GregorianCalendar();
      gC.setTime(new Date(System.currentTimeMillis()));

      header.setCreationTime(gC);

      header.setInstanceDescription("Shark's packages instance list");

      int itemsPerPage = config.getInt("WorkListHandler."
                                       + ParamConsts.PROPERTY_PREFIX_PROCESSINSTLIST
                                       + ".max.rows.per.page", 5);

      WfProcessMgrIterator pmgri = sc.get_iterator_processmgr();
      ProcessMgrFilterBuilder fb = Shark.getInstance().getProcessMgrFilterBuilder();
      WMFilter filter = fb.createEmptyFilter(sc.getSessionHandle());

      int hm = pmgri.how_many();
      int startAt = Integer.parseInt(from_element != null ? from_element : "0");
    
      if (startAt > hm) {
         startAt = (hm / itemsPerPage) * itemsPerPage;

         from_element = Integer.toString(startAt);

      } else if (startAt == hm && hm > 0) {
         startAt = startAt - itemsPerPage;
         from_element = Integer.toString(startAt);
      }

      from_element = Integer.toString(startAt);
      to_element = Integer.toString(startAt + itemsPerPage - 1);
      total_number = Integer.toString(hm);

      filter = fb.setLimit(sc.getSessionHandle(), filter, itemsPerPage);
      filter = fb.setStartPosition(sc.getSessionHandle(), filter, startAt);

      if (this.sortCriterion == null || this.sortCriterion.equals("")) { // search as
         // declared in
         // config file
         String orderByList = config.getString("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_PROCESSINSTLIST
                                               + ".fill.xpil.orderbylist", "");
         System.err.println("OBL="+orderByList);
         String[] obstrs = MiscUtilities.tokenize(orderByList, ";");
         if (obstrs.length > 0) {
            for (int i = 0; i < obstrs.length; i++) {
               String obs = obstrs[i];
               String[] sortinfo = MiscUtilities.tokenize(obs, ",");
               if (sortinfo != null && sortinfo.length == 2) {
                  String orderBy = sortinfo[0];
                  boolean asc = new Boolean(sortinfo[1]).booleanValue();
                  if (orderBy.equals("packageid")) {
                     fb.setOrderByPackageId(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("processdefid")) {
                     fb.setOrderByProcessDefId(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("version")) {
                     fb.setOrderByVersion(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("state")) {
                     fb.setOrderByEnabled(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("name")) {
                     fb.setOrderByName(sc.getSessionHandle(), filter, asc);
                  }
               }
            }
         }
      }
      // user has chosen search criterion
      // can search by state, createdTime, priority, requester, name
      else {
         if (this.sortCriterion.equalsIgnoreCase("packageid")) {
            fb.setOrderByPackageId(sc.getSessionHandle(),
                                   filter,
                                   Boolean.valueOf(this.sortAsc).booleanValue());
         } else if (this.sortCriterion.equalsIgnoreCase("processdefid")) {
            fb.setOrderByProcessDefId(sc.getSessionHandle(),
                                      filter,
                                      Boolean.valueOf(this.sortAsc).booleanValue());
         } else if (this.sortCriterion.equalsIgnoreCase("version")) {
            fb.setOrderByVersion(sc.getSessionHandle(),
                                 filter,
                                 Boolean.valueOf(this.sortAsc).booleanValue());
         } else if (this.sortCriterion.equalsIgnoreCase("state")) {
            fb.setOrderByEnabled(sc.getSessionHandle(),
                                 filter,
                                 Boolean.valueOf(this.sortAsc).booleanValue());
         } else if (this.sortCriterion.equalsIgnoreCase("name")) {
            fb.setOrderByName(sc.getSessionHandle(),
                              filter,
                              Boolean.valueOf(this.sortAsc).booleanValue());
         }
      }

      String exp = fb.toIteratorExpression(sc.getSessionHandle(), filter);
      pmgri.set_query_expression(exp);
      WfProcessMgr[] wfRs = pmgri.get_next_n_sequence(0);

      PackageInstances pis = ins.addNewPackageInstances();
      PackageInstance[] piArr = getPackageInstanceArr(sc, wfRs);

      pis.setPackageInstanceArray(piArr);

      return tws;
   }

   private PackageInstance[] getPackageInstanceArr(SharkConnection sc, WfProcessMgr[] wfRs)
      throws Exception {

      PackageInstance[] packageInstance = new PackageInstance[wfRs.length];

      AdminMisc am = Shark.getInstance().getAdminMisc();

      for (int i = 0; i < packageInstance.length; i++) {

         WorkflowProcessFactoryInstance[] wfp = new WorkflowProcessFactoryInstance[1];
         wfp[0] = SharkXPILBindingUtils.getProcessFactory(sc,
                                                          config,
                                                          "processinstlist",
                                                          (WfProcessMgr) wfRs[i]);

         PackageInstance pi = PackageInstance.Factory.newInstance();
         WMEntity pDefEnt = am.getProcessDefinitionInfoByUniqueProcessDefinitionName(sc.getSessionHandle(),
                                                                                     wfRs[i].name());
         pi.setId(pDefEnt.getPkgId());
         pi.setVersion(((WfProcessMgr) wfRs[i]).version());
         packageInstance[i] = pi;
         packageInstance[i].addNewWorkflowProcessFactoryInstances()
            .setWorkflowProcessFactoryInstanceArray(wfp);

      }
      return packageInstance;

   }

}
