package org.enhydra.wlh.business;

import java.sql.Date;
import java.util.Arrays;
import java.util.Comparator;
import java.util.GregorianCalendar;

import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILHeaderDocument.Header;
import org.enhydra.shark.xpil.XPILInstanceExtendedAttributeDocument.InstanceExtendedAttribute;
import org.enhydra.shark.xpil.XPILPackageInstancesDocument.PackageInstances;
import org.enhydra.wlh.spec.utils.ParamConsts;

import com.lutris.util.Config;

public class XMLRepositoryBuilder extends XMLBuilderImpl {

   public XMLRepositoryBuilder(String username,
                               Config config,
                               String startElement,
                               String sortCriterion,
                               String sortAsc) {

      this.username = username;
      this.config = config;
      this.from_element = startElement;
      this.sortCriterion = sortCriterion;
      this.sortAsc = sortAsc;

   }

   protected XPILExtendedWorkflowFacilityInstanceDocument fillXMLElements(SharkConnection sc)
      throws Exception {
      XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.newInstance();

      tws.addNewExtendedWorkflowFacilityInstance();

      ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();
      ins.addNewHeader();
      Header header = tws.getExtendedWorkflowFacilityInstance().getHeader();
      header.setXPILVersion("1.0");
      header.setXPILVendor("Shark");

      GregorianCalendar gC = new GregorianCalendar();
      gC.setTime(new Date(System.currentTimeMillis()));

      header.setCreationTime(gC);

      header.setInstanceDescription("Shark's packages instance list");

      int itemsPerPage = config.getInt("WorkListHandler."
                                       + ParamConsts.PROPERTY_PREFIX_REPOSITORY
                                       + ".max.rows.per.page", 5);

      PackageInstances pis = ins.addNewPackageInstances();

      RepositoryManager rm = RepositoryManager.getInstance();
      String xpdlRepository = config.getString("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_REPOSITORY
                                               + ".XPDL_repository");
      rm.setPathToXPDLRepositoryFolder(xpdlRepository);
      String[] paths = rm.getPackagePaths();
      
      Arrays.sort(paths,new RepositoryComparator(Boolean.valueOf( this.sortAsc ).booleanValue()));
      
     
      int hm = paths.length;
      int startAt = Integer.parseInt(from_element != null ? from_element : "0");

      if (startAt > hm) {
         startAt = (hm / itemsPerPage) * itemsPerPage;

         from_element = Integer.toString(startAt);

      } else if (startAt == hm && hm > 0) {
         startAt = startAt - itemsPerPage;
         from_element = Integer.toString(startAt);
      }

      from_element = Integer.toString(startAt);
      to_element = Integer.toString(startAt + itemsPerPage - 1);
      total_number = Integer.toString(hm);
      
      int endEl=hm-1;
      
      if(endEl>(startAt + itemsPerPage - 1)){
         endEl=(startAt + itemsPerPage - 1);
      }
 
      
      InstanceExtendedAttribute[] iea = new InstanceExtendedAttribute[endEl-startAt+1];
      int j=0;
      for (int i = startAt; i < endEl+1; i++) {
         InstanceExtendedAttribute ea = InstanceExtendedAttribute.Factory.newInstance();
        
         ea.setName(paths[i]);
         iea[j++] = (InstanceExtendedAttribute) ea;
      }
      pis.addNewInstanceExtendedAttributes().setInstanceExtendedAttributeArray(iea);

      return tws;
   }

}
class RepositoryComparator implements Comparator {

   protected boolean ascending;

   public RepositoryComparator(boolean asc) {
      this.ascending=asc;
   }

   public int compare(Object first, Object second) {
      if (first instanceof String && second instanceof String) {
         try {
            String str1 = (String) first;
            String str2 = (String) second;
            if (ascending) {
               return str1.compareTo(str2);
            }
            return str2.compareTo(str1);
         } catch (Exception ex) {
            // exception can happen if there's no value in sorted column
         }
      }

      return 1;
}
} 
