package org.enhydra.wlh.business;

import java.sql.Date;
import java.util.GregorianCalendar;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMFilter;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfAssignment;
import org.enhydra.shark.api.client.wfmodel.WfAssignmentIterator;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.common.AssignmentFilterBuilder;
import org.enhydra.shark.utilities.MiscUtilities;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILHeaderDocument.Header;
import org.enhydra.shark.xpil.XPILManualActivityInstanceDocument.ManualActivityInstance;
import org.enhydra.wlh.spec.utils.ParamConsts;

import com.lutris.util.Config;

public class XMLWorklistBuilder extends XMLBuilderImpl {
   
   protected String proId = null;

   protected String actId = null;

   public XMLWorklistBuilder(String username,
                             String proID,
                             String actID,
                             Config config,
                             String startElement, 
                             String sortCriterion, 
                             String sortAsc) {
      this.username = username;
      this.config = config;
      this.proId = proID;
      this.actId = actID;
      this.config = config;
      this.from_element = startElement;
      this.sortCriterion = sortCriterion;
      this.sortAsc = sortAsc;
   }

   protected XPILExtendedWorkflowFacilityInstanceDocument fillXMLElements(SharkConnection sc)
      throws Exception {
      XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.newInstance();

      tws.addNewExtendedWorkflowFacilityInstance();

      ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();
      ins.addNewHeader();
      Header header = tws.getExtendedWorkflowFacilityInstance().getHeader();
      header.setXPILVersion("1.0");
      header.setXPILVendor("Shark");

      GregorianCalendar gC = new GregorianCalendar();
      gC.setTime(new Date(System.currentTimeMillis()));

      header.setCreationTime(gC);

      header.setInstanceDescription("Shark's worklist for user " + username);

      int itemsPerPage = config.getInt("WorkListHandler."
                                       + ParamConsts.PROPERTY_PREFIX_WORKLIST
                                       + ".max.rows.per.page", 10);

      int startAt = Integer.parseInt(from_element != null ? from_element : "0");

      WfAssignmentIterator ait = sc.get_iterator_assignment();
      AssignmentFilterBuilder fb = Shark.getInstance().getAssignmentFilterBuilder();
      WMFilter filter = fb.addUsernameEquals(sc.getSessionHandle(), username);
      String exp = fb.toIteratorExpression(sc.getSessionHandle(), filter);
      ait.set_query_expression(exp);
      int hm = ait.how_many();
      if (startAt > hm) {
         startAt = (hm / itemsPerPage) * itemsPerPage;
      } else if (startAt == hm && hm > 0) {
         startAt = startAt - itemsPerPage;
      }
      from_element = Integer.toString(startAt);
      to_element = Integer.toString(startAt + itemsPerPage - 1);
      total_number = Integer.toString(hm);

      filter = fb.setLimit(sc.getSessionHandle(), filter, itemsPerPage);
      filter = fb.setStartPosition(sc.getSessionHandle(), filter, startAt);
      if(this.sortCriterion == null || this.sortCriterion.equals(""))  {
         String orderByList = config.getString("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_WORKLIST
                                               + ".fill.xpil.orderbylist", "");
         String[] obstrs = MiscUtilities.tokenize(orderByList, ";");
         if (obstrs.length > 0) {
            for (int i = 0; i < obstrs.length; i++) {
               String obs = obstrs[i];
               String[] sortinfo = MiscUtilities.tokenize(obs, ",");
               if (sortinfo != null && sortinfo.length == 2) {
                  String orderBy = sortinfo[0];
                  boolean asc = new Boolean(sortinfo[1]).booleanValue();
                  if (orderBy.equals("accepted")) {
                     fb.setOrderByAccepted(sc.getSessionHandle(), filter, asc);
                  } else if (orderBy.equals("createdtime")) {
                     fb.setOrderByCreatedTime(sc.getSessionHandle(), filter, asc);
                  }
               }
            }
         }
      }
      else {
         if(this.sortCriterion.equalsIgnoreCase("accepted")) {
            fb.setOrderByAccepted(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue() );
         }
         else if(this.sortCriterion.equalsIgnoreCase("createdtime")) {
            fb.setOrderByCreatedTime(sc.getSessionHandle(), filter, Boolean.valueOf( this.sortAsc ).booleanValue());
         }
      }
      

      exp = fb.toIteratorExpression(sc.getSessionHandle(), filter);
      ait.set_query_expression(exp);
      WfAssignment[] ass = ait.get_next_n_sequence(0);

      for (int i = 0; i < ass.length; i++) {
         WfActivity act = ass[i].activity();
         WfProcess pro = act.container();

         XPILWorkflowProcessInstance p = null;
         if (pro.requester() instanceof WfActivity) {
            p = ins.addNewSubWorkflowProcessInstance();
         } else {
            p = ins.addNewMainWorkflowProcessInstance();
         }
         SharkXPILBindingUtils.fillProcess(sc,
                                           config,
                                           ParamConsts.PROPERTY_PREFIX_WORKLIST,
                                           pro,
                                           p);
         ManualActivityInstance ma = SharkXPILBindingUtils.getManualActivity(sc,
                                                                             config,
                                                                             ParamConsts.PROPERTY_PREFIX_WORKLIST,
                                                                             act);
         p.addNewActivityInstances();
         p.getActivityInstances()
            .setManualActivityInstanceArray(new ManualActivityInstance[] {
               ma
            });
      }

      boolean pManagerList = config.getBoolean("WorkListHandler."
                                               + ParamConsts.PROPERTY_PREFIX_WORKLIST
                                               + ".fill.xpil.processfactories", true);
      if (pManagerList) {
         ins.setWorkflowProcessFactoryInstanceArray(SharkXPILBindingUtils.getProcessFactories(sc,
                                                                                              config,
                                                                                              ParamConsts.PROPERTY_PREFIX_WORKLIST));
      }

      // System.out.println("XMLWorklistBuilder = " + tws.toString() );

      return tws;

   }
}
