package org.enhydra.wlh.business;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPILBooleanDataInstanceDocument.BooleanDataInstance;
import org.enhydra.shark.xpil.XPILDateDataInstanceDocument.DateDataInstance;
import org.enhydra.shark.xpil.XPILDoubleDataInstanceDocument.DoubleDataInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILLongDataInstanceDocument.LongDataInstance;
import org.enhydra.shark.xpil.XPILManualActivityInstanceDocument.ManualActivityInstance;
import org.enhydra.shark.xpil.XPILSchemaDataInstanceDocument.SchemaDataInstance;
import org.enhydra.shark.xpil.XPILStringDataInstanceDocument.StringDataInstance;
import org.w3c.dom.Node;

public class XPILUtils {

   public static String extractProcessId(XPILExtendedWorkflowFacilityInstanceDocument xpil)
      throws Exception {

      String procId = null;

      XPILWorkflowProcessInstance[] wpia = xpil.getExtendedWorkflowFacilityInstance()
         .getMainWorkflowProcessInstanceArray();
      if (wpia == null) {
         wpia = xpil.getExtendedWorkflowFacilityInstance()
            .getSubWorkflowProcessInstanceArray();
      }
      XPILWorkflowProcessInstance proXpil = wpia[0];

      procId = proXpil.getId();
      return procId;
   }

   public static String extractProcessDescription(XPILExtendedWorkflowFacilityInstanceDocument xpil)
      throws Exception {

      String procDesc = null;

      XPILWorkflowProcessInstance[] wpia = xpil.getExtendedWorkflowFacilityInstance()
         .getMainWorkflowProcessInstanceArray();
      if (wpia == null) {
         wpia = xpil.getExtendedWorkflowFacilityInstance()
            .getSubWorkflowProcessInstanceArray();
      }
      XPILWorkflowProcessInstance proXpil = wpia[0];

      procDesc = proXpil.getInstanceDescription();
      return procDesc;
   }

   public static int extractProcessPriority(XPILExtendedWorkflowFacilityInstanceDocument xpil)
      throws Exception {

      int procPrio = 3;

      XPILWorkflowProcessInstance[] wpia = xpil.getExtendedWorkflowFacilityInstance()
         .getMainWorkflowProcessInstanceArray();
      if (wpia == null) {
         wpia = xpil.getExtendedWorkflowFacilityInstance()
            .getSubWorkflowProcessInstanceArray();
      }
      XPILWorkflowProcessInstance proXpil = wpia[0];

      procPrio = proXpil.getInstancePriority();
      return procPrio;
   }

   public static String extractActivityId(XPILExtendedWorkflowFacilityInstanceDocument xpil)
      throws Exception {

      String actId = null;

      ManualActivityInstance actXpil = xpil.getExtendedWorkflowFacilityInstance()
         .getManualActivityInstanceArray(0);
      actId = actXpil.getId();

      return actId;
   }

   public static Map extractVariablesFromXPIL(XPILExtendedWorkflowFacilityInstanceDocument xpil)
      throws Exception {

      Map map = new HashMap();

      ExtendedWorkflowFacilityInstance ins = xpil.getExtendedWorkflowFacilityInstance();

      StringDataInstance[] sIns = ins.getStringDataInstanceArray();
      for (int i = 0; i < sIns.length; i++) {
         StringDataInstance sIn = sIns[i];
         if (sIn.getValue() != null && !sIn.getValue().equals(""))
            map.put(sIn.getId(), sIn.getValue());
      }

      DoubleDataInstance[] dIns = ins.getDoubleDataInstanceArray();
      for (int i = 0; i < dIns.length; i++) {
         DoubleDataInstance dIn = dIns[i];

         map.put(dIn.getId(), new Double(dIn.getValue()));
      }
      BooleanDataInstance[] bIns = ins.getBooleanDataInstanceArray();
      for (int i = 0; i < bIns.length; i++) {
         BooleanDataInstance bIn = bIns[i];
         map.put(bIn.getId(), new Boolean(bIn.getValue()));
      }
      LongDataInstance[] lIns = ins.getLongDataInstanceArray();
      for (int i = 0; i < lIns.length; i++) {
         LongDataInstance lIn = lIns[i];
         map.put(lIn.getId(), new Long(lIn.getValue()));
      }
      DateDataInstance[] tIns = ins.getDateDataInstanceArray();
      for (int i = 0; i < tIns.length; i++) {
         DateDataInstance tIn = tIns[i];
         map.put(tIn.getId(), new Date(tIn.getValue().getTimeInMillis()));
      }

      SchemaDataInstance[] schIns = ins.getSchemaDataInstanceArray();
      for (int i = 0; i < schIns.length; i++) {
         SchemaDataInstance schIn = schIns[i];
         Node sn = schIn.getValue().getDomNode().getFirstChild().getNextSibling();
         if (sn == null) {
            sn = schIn.getValue().getDomNode().getFirstChild();
         }
         map.put(schIn.getId(), sn);
      }

      return map;

   }

}
