package org.enhydra.wlh.presentation;

import java.io.IOException;
import java.io.StringReader;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.SharkUtils;
import org.enhydra.wlh.business.XMLActivityBuilder;
import org.enhydra.wlh.business.XPILUtils;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.presentation.utils.XFormsGenerator;
import org.enhydra.wlh.spec.Log;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.io.OutputOptions;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;

public class ActivityHandlerPO extends BasePO {

   public Node handleDefault() throws HttpPresentationException {
      return showPage(null, null);
   }

   protected boolean loggedInUserRequired() {
      return true;
   }

   protected XMLObject showPage(String pro_Id, String act_Id)
      throws HttpPresentationException {

      EmptyXHTML htmlPage = (EmptyXHTML) myComms.xmlcFactory.create(EmptyXHTML.class);

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);

         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_ACTIVITYDETAILS + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         if (pro_Id == null) {
            pro_Id = myComms.request.getParameter(ParamConsts.PROCESS_ID);
            act_Id = myComms.request.getParameter(ParamConsts.ACTIVITY_ID);
         }

         switch (SharkUtils.getOutputType(getUsername(), pro_Id, act_Id)) {
            case 0:

               XFormsGenerator xform = new XFormsGenerator(getUsername(), pro_Id, act_Id);

               XMLObject retVal = xform.getXHTML();

               if (retVal != null)
                  return retVal;
               throw new Exception("Page can't be generate from XForms class");

            default:
               String from_element = myComms.request.getParameter("from");
               if (from_element == null || from_element.length() == 0) {
                  from_element = "0";
               }
               String sortCriterion = myComms.request.getParameter("sortCriterion");
               String sortOrder = myComms.request.getParameter("sortAsc");

               if (sortCriterion == null) {
                  sortCriterion = "";
               }
               String sortAsc = "true";
               if (sortOrder != null
                   && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
                  sortAsc = sortOrder;
               }
               XMLActivityBuilder xmlImp = new XMLActivityBuilder(getUsername(),
                                                                  pro_Id,
                                                                  act_Id,
                                                                  WorkListHandler.config,
                                                                  from_element,
                                                                  sortCriterion,
                                                                  sortAsc);

               String finall = null;
               if (!xslt.equals("none")) {

                  finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

               } else {

                  myComms.response.setContentType("text/xml");
                  HttpPresentationOutputStream os = myComms.response.getOutputStream();
                  xmlImp.writeResult(os);
                  myComms.response.flush();
                  return null;

               }

               DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
               dbf.setNamespaceAware(true);
               Document temp = dbf.newDocumentBuilder()
                  .parse(new InputSource(new StringReader(finall)));

               Node parent = htmlPage.getElementRemoveHtml().getParentNode();
               parent.replaceChild(htmlPage.importNode(temp.getFirstChild(), true),
                                   htmlPage.getElementRemoveHtml());

               return htmlPage;

         }

      } catch (Exception ex) {
         Log.logException(ex);
         throw new PresentationException("Can't show page", ex);
      }
   }

   public XMLObject handleUpdate() throws HttpPresentationException {

       String result = null;
       try {
          result = getRequestContentAsString(myComms.request.getHttpServletRequest());
       } catch (IOException ioex) {

       }
      // String result = myComms.request.getParameter("chiba.response");
      String from_element = myComms.request.getParameter("from");
      String sortCriterion = myComms.request.getParameter("sortCriterion");
      String sortOrder = myComms.request.getParameter("sortAsc");

      if (sortCriterion == null) {
         sortCriterion = "";
      }
      String sortAsc = "true";
      if (sortOrder != null
          && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
         sortAsc = sortOrder;
      }
      try {

         XPILExtendedWorkflowFacilityInstanceDocument xpil = XPILExtendedWorkflowFacilityInstanceDocument.Factory.parse(result);
         String procId = XPILUtils.extractProcessId(xpil);
         String actId = XPILUtils.extractActivityId(xpil);
         Map map = XPILUtils.extractVariablesFromXPIL(xpil);
         int proPri = XPILUtils.extractProcessPriority(xpil);
         String proDesc = XPILUtils.extractProcessDescription(xpil);

         SharkUtils.updateVariablesDescAndPriority(getUsername(),
                                                   procId,
                                                   actId,
                                                   map,
                                                   proDesc,
                                                   proPri);

      } catch (Exception e) {
         Log.logException(e);
         throw new PresentationException("Can't update", e);
      }

      throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                            + "?from=" + from_element + "&sortCriterion="
                                            + sortCriterion + "&sortAsc=" + sortAsc);

   }

   public XMLObject handleComplete() throws HttpPresentationException {

       String result = null;
       try {
          result = getRequestContentAsString(myComms.request.getHttpServletRequest());
       } catch (IOException ioex) {
           ioex.printStackTrace();
       }

      // String result = myComms.request.getParameter("chiba.response");
      String from_element = myComms.request.getParameter("from");
      String sortCriterion = myComms.request.getParameter("sortCriterion");
      String sortOrder = myComms.request.getParameter("sortAsc");

      if (sortCriterion == null) {
         sortCriterion = "";
      }
      String sortAsc = "true";
      if (sortOrder != null
          && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
         sortAsc = sortOrder;
      }
      String procId = null;

      try {

         XPILExtendedWorkflowFacilityInstanceDocument xpil = XPILExtendedWorkflowFacilityInstanceDocument.Factory.parse(result);
         procId = XPILUtils.extractProcessId(xpil);
         String actId = XPILUtils.extractActivityId(xpil);
         Map map = XPILUtils.extractVariablesFromXPIL(xpil);

         boolean cont = SharkUtils.continueProcess(getUsername(), procId, actId, map);
         boolean checkForContinuation = myComms.application.getConfig()
            .getBoolean("WorkListHandler."
                              + ParamConsts.PROPERTY_PREFIX_ACTIVITYDETAILS
                              + ".checkforcontinuation",
                        false);

         if (checkForContinuation) {
            if (cont) {
               SharkConnection sc = Shark.getInstance().getSharkConnection();
               WMConnectInfo wmci = new WMConnectInfo(getUsername(), "", "", "");
               sc.connect(wmci);
               WfProcess proc = sc.getProcess(procId);
               WfActivity toCont = SharkUtils.findNextActivity(proc, getUsername());
               if (toCont != null) {
                  toCont.change_state(SharkConstants.STATE_OPEN_RUNNING);
                  String qp = NavigConsts.ACTIVITY_HANDLER
                              + "?" + ParamConsts.PROCESS_ID + "=" + procId + "&"
                              + ParamConsts.ACTIVITY_ID + "=" + toCont.key() + "&from="
                              + from_element + "&sortCriterion=" + sortCriterion
                              + "&sortAsc=" + sortAsc;

                  throw new ClientPageRedirectException(qp);
               }
            }
            redirectAfterProcessEnd(procId);
         }

      } catch (Exception e) {
         Log.logException(e);
         throw new PresentationException("Can't complete", e);
      }

      throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                            + "?from=" + from_element + "&sortCriterion="
                                            + sortCriterion + "&sortAsc=" + sortAsc);

   }

   public XMLObject handleClose() throws HttpPresentationException {

      String from_element = myComms.request.getParameter("from");
      String sortCriterion = myComms.request.getParameter("sortCriterion");
      String sortOrder = myComms.request.getParameter("sortAsc");

      if (sortCriterion == null) {
         sortCriterion = "";
      }
      String sortAsc = "true";
      if (sortOrder != null
          && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
         sortAsc = sortOrder;
      }
      throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                            + "?from=" + from_element + "&sortCriterion="
                                            + sortCriterion + "&sortAsc=" + sortAsc);

   }

   protected void redirectAfterProcessEnd(String procId) throws HttpPresentationException {
      try {
         String from_element = myComms.request.getParameter("from");
         String pro_Id = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }
         String redirectTo = myComms.application.getConfig()
            .getString("WorkListHandler."
                             + ParamConsts.PROPERTY_PREFIX_ACTIVITYDETAILS
                             + ".RedirectAfterProcessEnd",
                       "WorklistHandlerPO.po");
         throw new ClientPageRedirectException(redirectTo
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc + "&"
                                               + ParamConsts.PROCESS_ID + "=" + procId);

      } catch (Exception ex) {
         throw new HttpPresentationException(ex);
      }
   }

   public OutputOptions getPostProcessingOptions() throws HttpPresentationException {

      OutputOptions oo = new OutputOptions();
      oo.setMIMEType("text/xhtml+xforms");
      oo.setEncoding("UTF-8");
      oo.addFreeformOption("session", myComms.session);
      oo.addFreeformOption("AdapterParamName",
                           myComms.request.getPresentationObjectRelativePath());
      return oo;
   }

}
