package org.enhydra.wlh.presentation;

import java.io.File;
import java.io.OutputStreamWriter;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfservice.PackageAdministration;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.XMLPackageManagementInstanceBuilder;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Node;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;

public class PackageHandlerPO extends BasePO {

   public Node handleDefault() throws HttpPresentationException {

      return showPage();
   }

   public XMLObject handleRefresh() throws HttpPresentationException {
      return showPage();
   }
   public XMLObject handleSort() throws HttpPresentationException {
      return showPage();
   }
   public XMLObject handleLoad() throws HttpPresentationException {
      try {
         String package_name = myComms.request.getParameter(ParamConsts.LOAD_PACKAGE_NAME);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }
         String xpdlRepository = myComms.application.getConfig()
            .getString("WorkListHandler."
                       + ParamConsts.PROPERTY_PREFIX_REPOSITORY + ".XPDL_repository");
         SharkConnection sc = Shark.getInstance().getSharkConnection();
         WMConnectInfo wmci = new WMConnectInfo(super.getUsername(), "", "", "");
         sc.connect(wmci);

         PackageAdministration pa = Shark.getInstance().getPackageAdministration();
         pa.openPackage(sc.getSessionHandle(), xpdlRepository
                                               + File.separator + package_name);
         sc.disconnect();

         throw new ClientPageRedirectException(NavigConsts.PACKAGE_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);
      } catch (Exception e) {
        throw new PresentationException("Problems during package opening!", e);
      }
   }

   public XMLObject handleUnload() throws HttpPresentationException {
      try {
         String package_name = myComms.request.getParameter(ParamConsts.DEL_NAME);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         SharkConnection sc = Shark.getInstance().getSharkConnection();
         WMConnectInfo wmci = new WMConnectInfo(super.getUsername(), "", "", "");
         sc.connect(wmci);
         PackageAdministration pa = Shark.getInstance().getPackageAdministration();
         String version = pa.getCurrentPackageVersion(sc.getSessionHandle(), package_name);
         pa.closePackage(sc.getSessionHandle(), package_name, version);
         sc.disconnect();
         throw new ClientPageRedirectException(NavigConsts.PACKAGE_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);
      } catch (Exception e) {
         throw new PresentationException("Problems during package closing!", e);
      }
   }

   public XMLObject handleUpdate() throws HttpPresentationException {

      String up_package_name = myComms.request.getParameter(ParamConsts.UPDATE_NAME);
      String from_element = myComms.request.getParameter("from");
      String sortCriterion = myComms.request.getParameter("sortCriterion");
      String sortOrder = myComms.request.getParameter("sortAsc");

      if (sortCriterion == null) {
         sortCriterion = "";
      }
      String sortAsc = "true";
      if (sortOrder != null
          && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
         sortAsc = sortOrder;
      }

      throw new ClientPageRedirectException(NavigConsts.UP_PACKAGE_HANDLER
                                            + "?from=" + from_element + "&sortCriterion="
                                            + sortCriterion + "&sortAsc=" + sortAsc
                                            + "&up_name=" + up_package_name);
   }

   protected XMLObject showPage() throws HttpPresentationException {

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);
         // xslt="none";
         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_PACKAGELIST + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         String pagingFrom = myComms.request.getParameter("from");

         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");
         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         XMLPackageManagementInstanceBuilder xmlImp = new XMLPackageManagementInstanceBuilder(getUsername(),
                                                                                              WorkListHandler.config,
                                                                                              pagingFrom,
                                                                                              sortCriterion,
                                                                                              sortAsc);
         String finall = null;
         if (!xslt.equals("none")) {

            finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

         } else {
            myComms.response.setContentType("text/xml");
            HttpPresentationOutputStream os = myComms.response.getOutputStream();
            xmlImp.writeResult(os);
            myComms.response.flush();
            return null;
         }

         HttpPresentationOutputStream os = myComms.response.getOutputStream();
         OutputStreamWriter out = new OutputStreamWriter(os);
         out.write(finall);
         out.flush();

         return null;
      } catch (Exception ex) {
         ex.printStackTrace();
         throw new PresentationException("Can't show page", ex);
      }
   }

   protected boolean loggedInUserRequired() {
      return true;
   }

}
