package org.enhydra.wlh.presentation;

import java.io.OutputStreamWriter;

import javax.transaction.UserTransaction;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfmodel.process_mgr_stateType;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.api.client.wfservice.WfProcessMgrIterator;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.SharkUtils;
import org.enhydra.wlh.business.XMLPackageManagementInstanceBuilder;
import org.enhydra.wlh.business.XMLProcessInstantiationBuilder;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Node;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;

public class ProcessInstantiationHandlerPO extends BasePO {

   public Node handleDefault() throws HttpPresentationException {
      return showPage();
   }

   public Node handleRefresh() throws HttpPresentationException {
      return showPage();
   }

   public XMLObject handleSort() throws HttpPresentationException {
      return showPage();
   }

   public Node handleEnable() throws HttpPresentationException {
      try {
         String procId = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String packageId = myComms.request.getParameter(ParamConsts.PACKAGE_ID);
         String version = myComms.request.getParameter(ParamConsts.VERSION);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }
         String wfpmId = packageId + "#" + version + "#" + procId;
         SharkConnection sc = Shark.getInstance().getSharkConnection();
         WMConnectInfo wmci = new WMConnectInfo(super.getUsername(), "", "", "");
         sc.connect(wmci);
         WfProcessMgr wfpm = sc.getProcessMgr(wfpmId);

         wfpm.set_process_mgr_state(process_mgr_stateType.enabled);

         sc.disconnect();
         throw new ClientPageRedirectException(NavigConsts.PROCESS_INST_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);

      } catch (Exception e) {
         e.printStackTrace();
         throw new PresentationException("Problems with changing state", e);

      }
   }

   public Node handleDisable() throws HttpPresentationException {

      try {

         String procId = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String packageId = myComms.request.getParameter(ParamConsts.PACKAGE_ID);
         String version = myComms.request.getParameter(ParamConsts.VERSION);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }
         String wfpmId = packageId + "#" + version + "#" + procId;
         SharkConnection sc = Shark.getInstance().getSharkConnection();
         WMConnectInfo wmci = new WMConnectInfo(super.getUsername(), "", "", "");
         sc.connect(wmci);

         WfProcessMgr wfpm = sc.getProcessMgr(wfpmId);

         wfpm.set_process_mgr_state(process_mgr_stateType.disabled);

         sc.disconnect();
         throw new ClientPageRedirectException(NavigConsts.PROCESS_INST_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc + "&pro_id="
                                               + procId + "&package_id=" + packageId
                                               + "&version=" + version);

      } catch (Exception e) {
         e.printStackTrace();
         throw new PresentationException("Problems with changing state", e);

      }
   }

   protected XMLObject showPage() throws HttpPresentationException {

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);
         // xslt="none";
         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_PROCESSINSTLIST + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         String pagingFrom = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         XMLProcessInstantiationBuilder xmlImp = new XMLProcessInstantiationBuilder(getUsername(),
                                                                                    WorkListHandler.config,
                                                                                    pagingFrom,
                                                                                    sortCriterion,
                                                                                    sortAsc);
         String finall = null;
         if (!xslt.equals("none")) {

            finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

         } else {
            myComms.response.setContentType("text/xml");
            HttpPresentationOutputStream os = myComms.response.getOutputStream();
            xmlImp.writeResult(os);
            myComms.response.flush();
            return null;
         }

         HttpPresentationOutputStream os = myComms.response.getOutputStream();
         OutputStreamWriter out = new OutputStreamWriter(os);
         out.write(finall);
         out.flush();

         return null;
      } catch (Exception ex) {
         ex.printStackTrace();
         throw new PresentationException("Can't show page", ex);
      }
   }

   protected boolean loggedInUserRequired() {
      // TODO Auto-generated method stub
      return false;
   }

   public XMLObject handleCreate() throws HttpPresentationException {

      try {
         String procId = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String packageId = myComms.request.getParameter(ParamConsts.PACKAGE_ID);
         String version = myComms.request.getParameter(ParamConsts.VERSION);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         boolean checkFirstActivity = myComms.application.getConfig()
            .getBoolean("WorkListHandler."
                              + ParamConsts.PROPERTY_PREFIX_PROCESSLIST
                              + ".checkforfirstactivity",
                        false);
         String process_manager_name = packageId + "#" + version + "#" + procId;
         WfActivity act = SharkUtils.createProcess(getUsername(),
                                                   process_manager_name,
                                                   checkFirstActivity);
         if (act != null) {
            act.change_state(SharkConstants.STATE_OPEN_RUNNING);
            String qp = NavigConsts.ACTIVITY_HANDLER
                        + "?" + ParamConsts.PROCESS_ID + "=" + act.container().key()
                        + "&" + ParamConsts.ACTIVITY_ID + "=" + act.key();
            throw new ClientPageRedirectException(qp);
         }

         String pageRedirect = myComms.application.getConfig()
            .getString("WorkListHandler."
                       + ParamConsts.PROPERTY_PREFIX_PROCESSINSTLIST + ".page.redirect");
         if (pageRedirect==""){
            throw new ClientPageRedirectException(NavigConsts.PROCESS_INST_HANDLER
                                                  + "?from=" + from_element
                                                  + "&sortCriterion=" + sortCriterion
                                                  + "&sortAsc=" + sortAsc);
         }
         else{
          throw new ClientPageRedirectException(pageRedirect);
         }
      } catch (Exception e) {
         throw new PresentationException("Problems with process creation!", e);
      }
   }

}
