package org.enhydra.wlh.presentation;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.RepositoryManager;
import org.enhydra.wlh.business.XMLRepositoryBuilder;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Node;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;
import com.lutris.mime.ContentHeader;
import com.lutris.mime.MimeHeader;
import com.lutris.mime.MultipartMimeInput;
import com.lutris.mime.MultipartMimeInputStream;

public class RepositoryHandlerPO extends BasePO {

   public Node handleDefault() throws HttpPresentationException {

      return showPage();
   }

   public Node handleRefresh() throws HttpPresentationException {

      return showPage();
   }
   public XMLObject handleSort() throws HttpPresentationException {
      return showPage();
   }
   public Node handleDelete() throws HttpPresentationException {

      String name = myComms.request.getParameter(ParamConsts.DEL_NAME);
      String from_element = myComms.request.getParameter("from");
      String sortCriterion = myComms.request.getParameter("sortCriterion");
      String sortOrder = myComms.request.getParameter("sortAsc");
      try {
         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         RepositoryManager rm = RepositoryManager.getInstance();

         String xpdlRepository;

         xpdlRepository = myComms.application.getConfig()
            .getString("WorkListHandler."
                       + ParamConsts.PROPERTY_PREFIX_REPOSITORY + ".XPDL_repository");
         rm.setPathToXPDLRepositoryFolder(xpdlRepository);
         rm.deletePackage(name);

         throw new ClientPageRedirectException(NavigConsts.REPOSITORY_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc + "&name=" + name);
      } catch (Exception e) {
         e.printStackTrace();
         throw new PresentationException("Problems with delete ", e);
      }
   }

   protected XMLObject showPage() throws HttpPresentationException {

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);
         // xslt = "none";
         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_REPOSITORY + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         String pagingFrom = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         XMLRepositoryBuilder xmlImp = new XMLRepositoryBuilder(getUsername(),
                                                                WorkListHandler.config,
                                                                pagingFrom,
                                                                sortCriterion,
                                                                sortAsc);
         String finall = null;
         if (!xslt.equals("none")) {

            finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

         } else {
            myComms.response.setContentType("text/xml");
            HttpPresentationOutputStream os = myComms.response.getOutputStream();
            xmlImp.writeResult(os);
            myComms.response.flush();
            return null;
         }

         HttpPresentationOutputStream os = myComms.response.getOutputStream();
         OutputStreamWriter out = new OutputStreamWriter(os);
         out.write(finall);
         out.flush();

         return null;
      } catch (Exception ex) {
         ex.printStackTrace();
         throw new PresentationException("Can't show page", ex);
      }
   }

   public Node handleUpload() throws PresentationException {
      try {
         String header = myComms.request.getHeader(ParamConsts.CONTENT_TYPE);
         if (header != null && !"".equals(header)) {
            ContentHeader contentHeader = new ContentHeader(header);
            MultipartMimeInput mInput = new MultipartMimeInput(myComms.request.getInputStream(),
                                                               contentHeader);

            String xpdlRepository = myComms.application.getConfig()
            .getString("WorkListHandler."
                       + ParamConsts.PROPERTY_PREFIX_REPOSITORY
                       + ".XPDL_repository");
            MultipartMimeInputStream mime = mInput.nextPart();
            while (mime != null) {
               MimeHeader mh = mime.getHeader(ParamConsts.CONTENT_DISPOSITION);
               String paramName = mh.getParameter("name");
               if (("contextFilePath").equals(paramName)) {
                  String xpdlName = mh.getParameter("filename");
                  String xpdlPath = getFile(mime, xpdlRepository, xpdlName);
               }
               mime = mInput.nextPart();
            }

         }

         throw new ClientPageRedirectException(NavigConsts.REPOSITORY_HANDLER);
      } catch (Exception ex) {

         throw new PresentationException("Error while deploy product", ex);

      }

   }

   private String getFile(MultipartMimeInputStream part, String tempPath, String warName)
      throws IOException {
      // Get the tempdir
      
      byte[] buf = new byte[1024];
      BufferedOutputStream bos = null;
      FileOutputStream fos = null;
      File file = null;
      try {
         File tmpfile = new File(warName);
         file = new File(tempPath + "/" + tmpfile.getName());
         fos = new FileOutputStream(file);
         bos = new BufferedOutputStream(fos);

         int b = 0;
         while ((b = part.read(buf, 0, buf.length)) != -1) {
            bos.write(buf, 0, b);
         }
         bos.flush();

      } catch (Exception ex) {
         file = null;
      } finally {
         if (fos != null)
            fos.close();
         if (bos != null)
            bos.close();
      }
      if (file != null)
         return file.getAbsolutePath();
      else
         return "";
   }

   protected boolean loggedInUserRequired() {
      // TODO Auto-generated method stub
      return false;
   }

}
