package org.enhydra.wlh.presentation;

import java.io.File;
import java.io.OutputStreamWriter;
import java.util.Map;

import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfservice.PackageAdministration;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.RepositoryManager;
import org.enhydra.wlh.business.XMLPackageUpdateBuilder;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Node;

import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;

public class UpdatePackageHandlerPO extends BasePO {

   public Node handleDefault() throws HttpPresentationException {
      return showPage();
   }

   public Node handleClose() throws HttpPresentationException {
      throw new ClientPageRedirectException(NavigConsts.PACKAGE_HANDLER);
   }
  
   public Node handleUpdate() throws HttpPresentationException {

      String uPackage = myComms.request.getParameter("up_name");
      RepositoryManager rm = RepositoryManager.getInstance();
      String xpdlRepository;
      try {
         xpdlRepository = myComms.application.getConfig()
            .getString("WorkListHandler."
                       + ParamConsts.PROPERTY_PREFIX_REPOSITORY + ".XPDL_repository");

         rm.setPathToXPDLRepositoryFolder(xpdlRepository);

         Map idMap = rm.getPackagePathToIdMapping();
         String packageId = idMap.get(uPackage).toString();

         SharkConnection sc = Shark.getInstance().getSharkConnection();
         WMConnectInfo wmci = new WMConnectInfo(super.getUsername(), "", "", "");
         sc.connect(wmci);

         PackageAdministration pa = Shark.getInstance().getPackageAdministration();
         pa.updatePackage(sc.getSessionHandle(), packageId, xpdlRepository
                                                            + File.separator + uPackage);

      } catch (Exception e) {

         e.printStackTrace();
      }
      throw new ClientPageRedirectException(NavigConsts.PACKAGE_HANDLER);
   }

   protected XMLObject showPage() throws HttpPresentationException {

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);
         // xslt="none";
         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_UP_PACKAGELIST + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         String pagingFrom = myComms.request.getParameter("from");
         String uPackage = myComms.request.getParameter("up_name");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");
         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         XMLPackageUpdateBuilder xmlImp = new XMLPackageUpdateBuilder(getUsername(),
                                                                      WorkListHandler.config,
                                                                      pagingFrom,
                                                                      sortCriterion,
                                                                      sortAsc,
                                                                      uPackage);
         String finall = null;
         if (!xslt.equals("none")) {

            finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

         } else {
            myComms.response.setContentType("text/xml");
            HttpPresentationOutputStream os = myComms.response.getOutputStream();
            xmlImp.writeResult(os);
            myComms.response.flush();
            return null;
         }

         HttpPresentationOutputStream os = myComms.response.getOutputStream();
         OutputStreamWriter out = new OutputStreamWriter(os);
         out.write(finall);
         out.flush();

         return null;
      } catch (Exception ex) {
         ex.printStackTrace();
         throw new PresentationException("Can't show page", ex);
      }
   }

   protected boolean loggedInUserRequired() {
      // TODO Auto-generated method stub
      return false;
   }

}
