package org.enhydra.wlh.presentation;

import java.io.OutputStreamWriter;

import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.common.SharkConstants;
import org.enhydra.wlh.WorkListHandler;
import org.enhydra.wlh.business.SharkUtils;
import org.enhydra.wlh.business.XMLWorklistBuilder;
import org.enhydra.wlh.presentation.utils.NavigConsts;
import org.enhydra.wlh.spec.utils.ParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.w3c.dom.Node;
import com.lutris.appserver.server.httpPresentation.ClientPageRedirectException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationException;
import com.lutris.appserver.server.httpPresentation.HttpPresentationOutputStream;

public class WorklistHandlerPO extends BasePO {

   public void testNodes() {

   }

   public Node handleDefault() throws HttpPresentationException {

      return showPage();

   }

   public XMLObject handleRefresh() throws HttpPresentationException {
      return showPage();
   }

   public XMLObject handleLogout() throws HttpPresentationException {
      myComms.request.getHttpServletRequest().getSession().invalidate();
      throw new ClientPageRedirectException("WorklistHandlerPO.po");
   }

   public XMLObject handleSort() throws HttpPresentationException {
      return showPage();
   }

   protected boolean loggedInUserRequired() {
      return true;
   }

   protected XMLObject showPage() throws HttpPresentationException {

      try {

         String xslt = myComms.request.getParameter(ParamConsts.XSL);

         if (xslt == null) {
            xslt = myComms.application.getConfig()
               .getString("WorkListHandler."
                                + ParamConsts.PROPERTY_PREFIX_WORKLIST + ".xsl",
                          "none");
         }

         String xslDir = WorkListHandler.getXslDirectory();

         String pro_Id = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String act_Id = myComms.request.getParameter(ParamConsts.ACTIVITY_ID);
         String pagingFrom = myComms.request.getParameter("from");

         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if(sortCriterion==null) {
            sortCriterion="";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         XMLWorklistBuilder xmlImp = new XMLWorklistBuilder(getUsername(),
                                                            pro_Id,
                                                            act_Id,
                                                            WorkListHandler.config,
                                                            pagingFrom,
                                                            sortCriterion,
                                                            sortAsc);
         String finall = null;
         if (!xslt.equals("none")) {

            finall = xmlImp.getResult(xslDir + "/" + xslt + ".xsl");

         } else {
            myComms.response.setContentType("text/xml");
            HttpPresentationOutputStream os = myComms.response.getOutputStream();
            xmlImp.writeResult(os);
            myComms.response.flush();
            return null;
         }

         HttpPresentationOutputStream os = myComms.response.getOutputStream();
         OutputStreamWriter out = new OutputStreamWriter(os);
         out.write(finall);
         out.flush();

         return null;
      } catch (Exception ex) {
         ex.printStackTrace();
         throw new PresentationException("Can't show page", ex);
      }
   }

   public XMLObject handleAccepted() throws HttpPresentationException {

      try {
         String procId = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String actId = myComms.request.getParameter(ParamConsts.ACTIVITY_ID);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         String accepted = myComms.request.getParameter(ParamConsts.ACCEPTED);
         SharkUtils.acceptedActivity(getUsername(), procId, actId, accepted);
         throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);

      } catch (Exception e) {
         e.printStackTrace();
         throw new PresentationException("Problems with accepting activity", e);

      }
   }

   public XMLObject handleCreate() throws HttpPresentationException {

      try {
         String process_manager_name = myComms.request.getParameter(ParamConsts.PROCESS_MANAGER);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         boolean checkFirstActivity = myComms.application.getConfig()
            .getBoolean("WorkListHandler."
                              + ParamConsts.PROPERTY_PREFIX_WORKLIST
                              + ".checkforfirstactivity",
                        false);

         WfActivity act = SharkUtils.createProcess(getUsername(),
                                                   process_manager_name,
                                                   checkFirstActivity);
         if (act != null) {
            act.change_state(SharkConstants.STATE_OPEN_RUNNING);
            String qp = NavigConsts.ACTIVITY_HANDLER
                        + "?" + ParamConsts.PROCESS_ID + "=" + act.container().key()
                        + "&" + ParamConsts.ACTIVITY_ID + "=" + act.key() + "&from=0"
                        + "&sortCriterion=" + sortCriterion + "&sortAsc=" + sortAsc;

            throw new ClientPageRedirectException(qp);
         }
         throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);
      } catch (Exception e) {
         throw new PresentationException("Problems with process creation!", e);
      }

   }

   public XMLObject handleComplete() throws HttpPresentationException {

      try {

         String procId = myComms.request.getParameter(ParamConsts.PROCESS_ID);
         String actId = myComms.request.getParameter(ParamConsts.ACTIVITY_ID);
         String from_element = myComms.request.getParameter("from");
         String sortCriterion = myComms.request.getParameter("sortCriterion");
         String sortOrder = myComms.request.getParameter("sortAsc");

         if (sortCriterion == null) {
            sortCriterion = "";
         }
         String sortAsc = "true";
         if (sortOrder != null
             && (sortOrder.equalsIgnoreCase("true") || sortOrder.equalsIgnoreCase("false"))) {
            sortAsc = sortOrder;
         }

         if (actId != null && !actId.equals("") && procId != null && !procId.equals("")) {
            boolean checkActivityContextUpdate = myComms.application.getConfig()
               .getBoolean("WorkListHandler."
                                 + ParamConsts.PROPERTY_PREFIX_WORKLIST
                                 + ".checkactivitycontextoncompletion",
                           false);

            boolean isContextUpdated = true;
            if (checkActivityContextUpdate) {
               isContextUpdated = SharkUtils.isContextUpdated(getUsername(),
                                                              procId,
                                                              actId);
            }
            if (isContextUpdated) {
               SharkUtils.completeActivity(getUsername(), procId, actId);
            } else {
               String qp = NavigConsts.ACTIVITY_HANDLER
                           + "?" + ParamConsts.PROCESS_ID + "=" + procId + "&"
                           + ParamConsts.ACTIVITY_ID + "=" + actId + "&from=0";
               throw new ClientPageRedirectException(qp);

            }
         }
         throw new ClientPageRedirectException(NavigConsts.WORKLIST_HANDLER
                                               + "?from=" + from_element
                                               + "&sortCriterion=" + sortCriterion
                                               + "&sortAsc=" + sortAsc);

      } catch (Exception e) {

         throw new PresentationException("Error wile completing activity", e);
      }

   }
}
