package org.enhydra.wlh.presentation.utils;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.xmlbeans.XmlDate;
import org.apache.xmlbeans.XmlObject;
import org.enhydra.shark.Shark;
import org.enhydra.shark.api.client.wfmc.wapi.WMConnectInfo;
import org.enhydra.shark.api.client.wfmodel.WfActivity;
import org.enhydra.shark.api.client.wfmodel.WfProcess;
import org.enhydra.shark.api.client.wfmodel.WfProcessMgr;
import org.enhydra.shark.api.client.wfservice.AdminMisc;
import org.enhydra.shark.api.client.wfservice.SharkConnection;
import org.enhydra.shark.api.client.wfservice.WMEntity;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument;
import org.enhydra.shark.xpil.XPILBooleanDataInstanceDocument.BooleanDataInstance;
import org.enhydra.shark.xpil.XPILDateDataInstanceDocument.DateDataInstance;
import org.enhydra.shark.xpil.XPILDoubleDataInstanceDocument.DoubleDataInstance;
import org.enhydra.shark.xpil.XPILExtendedWorkflowFacilityInstanceDocument.ExtendedWorkflowFacilityInstance;
import org.enhydra.shark.xpil.XPILLongDataInstanceDocument.LongDataInstance;
import org.enhydra.shark.xpil.XPILMainWorkflowProcessInstanceDocument.MainWorkflowProcessInstance;
import org.enhydra.shark.xpil.XPILManualActivityInstanceDocument.ManualActivityInstance;
import org.enhydra.shark.xpil.XPILSchemaDataInstanceDocument.SchemaDataInstance;
import org.enhydra.shark.xpil.XPILStringDataInstanceDocument.StringDataInstance;
import org.enhydra.wlh.business.SharkUtils;
import org.enhydra.wlh.business.VariableData;
import org.enhydra.wlh.spec.SharkParamConsts;
import org.enhydra.xml.xmlc.XMLObject;
import org.enhydra.xml.xmlc.html.HTMLObjectImpl;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.html.HTMLElement;

public class XFormsGenerator {

   private String username = null;

   private String actId = null;

   private String procId = null;

   public XFormsGenerator(String username, String processId, String activityId) {
      this.username = username;
      this.actId = activityId;
      this.procId = processId;

   }

   public XMLObject getXHTML() throws Exception {

      SharkConnection sc = Shark.getInstance().getSharkConnection();

      try {

         WMConnectInfo wmci = new WMConnectInfo(username, "", "", "");

         sc.connect(wmci);

         WfActivity act = sc.getActivity(procId, actId);

         AdminMisc am = Shark.getInstance().getAdminMisc();

         WfProcess pro = act.container();

         Map varMap = act.process_context();
         WfProcessMgr mgr = pro.manager();
         Map cs = mgr.context_signature();

         WMEntity varEnt = am.getActivityDefinitionInfo(sc.getSessionHandle(),
                                                        procId,
                                                        actId);

         String[][] extAttrbs = SharkUtils.getExtAttribNVPairs(sc.getSessionHandle(),
                                                               varEnt);

         if (SharkUtils.doesXFormExist(extAttrbs)) {

            List list = SharkUtils.classifyVariables(varMap, cs);
            return filledXForms(list, extAttrbs);

         }

         return null;

      } finally {

         sc.disconnect();

      }

   }

   private HTMLObjectImpl filledXForms(List list, String[][] extAttribs) throws Exception {
      HTMLObjectImpl template = null;

      for (int i = 0; i < extAttribs.length; i++) {
         String eaName = extAttribs[i][0];

         if (eaName.equals(SharkParamConsts.XFORMS_XMLC_CLASS)) {

            String tmplName = extAttribs[i][1];
            template = (HTMLObjectImpl) Class.forName(tmplName).newInstance();
            break;
         }
      }

      if (template != null) {

         HTMLElement xFormsInstance = null;
         NodeList nl = template.getElementsByTagName("xforms:instance");
//         System.err.println("Found " + nl.getLength() + " nodes xformsinstance");

         if (nl.getLength() > 0) {
            xFormsInstance = (HTMLElement) nl.item(0);
         }

         if (xFormsInstance == null) {
            Method method = template.getClass().getMethod("getElementInst1", null);
            xFormsInstance = (HTMLElement) method.invoke(template, null);
         }

         Node node = xFormsInstance.getFirstChild().getNextSibling();

         XPILExtendedWorkflowFacilityInstanceDocument tws = XPILExtendedWorkflowFacilityInstanceDocument.Factory.parse(node);

         ExtendedWorkflowFacilityInstance ins = tws.getExtendedWorkflowFacilityInstance();

         MainWorkflowProcessInstance[] proces = ins.getMainWorkflowProcessInstanceArray();

         proces[0].setId(procId);

         ManualActivityInstance[] accts = ins.getManualActivityInstanceArray();

         accts[0].setId(actId);

         StringDataInstance[] sIns = ins.getStringDataInstanceArray();
         for (int i = 0; i < sIns.length; i++) {
            StringDataInstance sIn = sIns[i];

            Iterator iter = list.iterator();

            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(sIn.getId())
                   && !varData.getId().equals(SharkParamConsts.NAVIGATION_VARIABLE)) {
                  sIn.setValue((String) varData.getValue());

               }

            }
         }

         DoubleDataInstance[] dIns = ins.getDoubleDataInstanceArray();
         for (int i = 0; i < dIns.length; i++) {
            DoubleDataInstance dIn = dIns[i];
            Iterator iter = list.iterator();

            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(dIn.getId())) {
                  dIn.setValue(((Double) varData.getValue()).doubleValue());

               }

            }
         }
         BooleanDataInstance[] bIns = ins.getBooleanDataInstanceArray();
         for (int i = 0; i < bIns.length; i++) {
            BooleanDataInstance bIn = bIns[i];
            Iterator iter = list.iterator();
            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(bIn.getId())) {
                  bIn.setValue(((Boolean) varData.getValue()).booleanValue());

               }

            }
         }
         LongDataInstance[] lIns = ins.getLongDataInstanceArray();
         for (int i = 0; i < lIns.length; i++) {
            LongDataInstance lIn = lIns[i];
            Iterator iter = list.iterator();

            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(lIn.getId())) {
                  lIn.setValue(((Long) varData.getValue()).longValue());

               }

            }
         }

         DateDataInstance[] tIns = ins.getDateDataInstanceArray();
         for (int i = 0; i < tIns.length; i++) {
            DateDataInstance tIn = tIns[i];
            Iterator iter = list.iterator();

            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(tIn.getId())) {
                  java.util.Date date = (java.util.Date) varData.getValue();
                  XmlDate a = XmlDate.Factory.newInstance();
                  SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd");
                  a.setStringValue(f.format(date));
                  tIn.xsetValue(a);                  
               }

            }
         }
         SchemaDataInstance[] schIns = ins.getSchemaDataInstanceArray();
         for (int i = 0; i < schIns.length; i++) {
            SchemaDataInstance schIn = schIns[i];
            Iterator iter = list.iterator();

            while (iter.hasNext()) {
               VariableData varData = (VariableData) iter.next();

               if (varData.getId().equals(schIn.getId())) {
                  XmlObject xmlO = XmlObject.Factory.parse((Node) varData.getValue());
                  schIn.setValue(xmlO);
               }

            }
         }

         xFormsInstance.replaceChild(template.importNode(ins.getDomNode(), true),
                                     xFormsInstance.getFirstChild().getNextSibling());
      }

      return template;
   }

}
