package org.enhydra.wlh.spec;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.util.GregorianCalendar;

import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.Logger;

/**
 * Class used for logging messages to file.
 */
public class Log {

   /**
    * When set to 'true' , this class will not log into file.
    */
   private static boolean disabled = false;

   private static String getLogFile() {
      return ('/' == File.separatorChar ? "/tmp" : "") + "/testchiba.log";
   }

   /**
    * Logs message to default file.
    * 
    * @param message message for logging,
    */
   public static void logToFile(String message) {
      Enhydra.getLogChannel().write(Logger.INFO, message);

   }

   public static void log(String message) {

      Enhydra.getLogChannel().write(Logger.INFO, message);
   }

   /**
    * Logs message to file.
    * 
    * @param fileLog file path and name which is used for logging.
    * @param message message for logging,
    */
   public static void logToFile(String fileLog, String message) {
      try {
         if (!disabled) {
            File file = new File(fileLog);
            if (!file.exists()) {
               file.createNewFile();
            }
            RandomAccessFile fileLogr = new RandomAccessFile(fileLog, "rw");
            fileLogr.seek(fileLogr.length());
            fileLogr.writeBytes(getTotalTime() + message + "\r\n");
            fileLogr.close();
         }
      } catch (Exception ex) {
         ex.printStackTrace();
      }
   }

   public static void logException(Throwable e) {
      if (!disabled) {
         e.printStackTrace();
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         e.printStackTrace(new PrintStream(out));
         Enhydra.getLogChannel().write(Logger.CRITICAL, out.toString());

      }
   }

   private static String getTotalTime() {
      StringBuffer time = new StringBuffer();
      time.append("[");
      GregorianCalendar cal = new GregorianCalendar();
      time.append(cal.get(GregorianCalendar.HOUR_OF_DAY));
      time.append(":");
      time.append(cal.get(GregorianCalendar.MINUTE));
      time.append(":");
      time.append(cal.get(GregorianCalendar.SECOND));
      time.append("]");
      time.append("  ");
      return time.toString();
   }
}
